<?php 
/*
	Plugin Name: Blackhole Pro
	Plugin URI: https://plugin-planet.com/blackhole-pro/
	Description: Protects your site against bad bots by trapping them in a virtual black hole.
	Tags: bots, blackhole, honeypot, security, anti-spam
	Author: Jeff Starr
	Contributors: specialk
	Author URI: https://monzillamedia.com/
	Donate link: https://monzillamedia.com/donate.html
	Requires at least: 4.6
	Tested up to: 6.5
	Stable tag: 3.4.1
	Version:    3.4.1
	Requires PHP: 5.6.20
	Text Domain: blackhole-pro
	Domain Path: /languages
	
	License: The Blackhole Pro license comprises two parts:
		
		* Part 1: Its PHP code is licensed under the GPL (v3 or later), like WordPress. More info @ https://www.gnu.org/licenses/
		
		* Part 2: Everything else (e.g., CSS, HTML, JavaScript, images, design) is licensed according to the purchased license. More info @ https://plugin-planet.com/blackhole-pro/
		
	Without prior written consent from Monzilla Media, you must NOT directly or indirectly: license, sub-license, sell, resell, or provide for free any aspect or component of Part 2.
	
	Further license information is available in the plugin directory, /license/, and online @ https://plugin-planet.com/wp/files/blackhole-pro/license.txt
	
	Upgrades: Your purchase of Blackhole Pro includes free lifetime upgrades, which include new features, bug fixes, and other improvements. 
	
	Copyright 2024 Monzilla Media. All rights reserved.
*/

if (!defined('ABSPATH')) die();
update_option( 'blackhole_pro_license_key', 'activated' );
update_option( 'blackhole_pro_license_status', 'valid' );
if (!class_exists('Blackhole_Pro')) {
	
	final class Blackhole_Pro {
		
		private static $instance;
		
		public static function instance() {
			if (!isset(self::$instance) && !(self::$instance instanceof Blackhole_Pro)) {
				
				self::$instance = new Blackhole_Pro;
				self::$instance->constants();
				self::$instance->includes();
				
				register_activation_hook(__FILE__, 'blackhole_dismiss_notice_activate');
				register_activation_hook(__FILE__, array(self::$instance, 'import_data'));
				register_deactivation_hook(__FILE__, 'blackhole_cron_deactivation');
				
				add_action('admin_init',          array(self::$instance, 'check_blackhole'));
				add_action('admin_init',          array(self::$instance, 'check_version'));
				add_action('init',                array(self::$instance, 'load_i18n'));
				add_filter('plugin_action_links', array(self::$instance, 'action_links'), 10, 2);
				add_filter('plugin_row_meta',     array(self::$instance, 'plugin_links'), 10, 2);
				add_filter('admin_footer_text',   array(self::$instance, 'footer_text'),  10, 1);
				add_filter('safe_style_css',      array(self::$instance, 'blackhole_allow_style'));
				
				add_action('admin_enqueue_scripts', 'blackhole_enqueue_resources_admin');
				add_action('admin_print_scripts',   'blackhole_print_js_vars_admin');
				add_action('admin_notices',         'blackhole_admin_notices');
				add_action('admin_menu',            'blackhole_menu_pages');
				add_action('admin_init',            'blackhole_dismiss_notice_save');
				add_action('admin_init',            'blackhole_dismiss_notice_version');
				add_action('admin_init',            'blackhole_register_settings');
				add_action('admin_init',            'blackhole_reset_options');
				add_action('admin_init',            'blackhole_reset_reports');
				add_action('admin_init',            'blackhole_reset_badbots');
				add_action('admin_init',            'blackhole_refresh_geo');
				add_action('admin_init',            'blackhole_delete_bot');
				add_action('admin_init',            'blackhole_add_bot');
				
				add_action('login_footer',           'blackhole_trigger');
				add_action(blackhole_trigger_hook(), 'blackhole_trigger');
				add_action('init',                   'blackhole_scanner');
				
				add_filter('robots_txt', 'blackhole_robots_wordpress', 11, 2);
				
				add_shortcode('blackhole_nonce',   'blackhole_trigger_nonce');
				add_shortcode('blackhole_trigger', 'blackhole_trigger_shortcode');
				
				add_action('blackhole_send_email_cron', 'blackhole_send_email_cron');
				
			}
			return self::$instance;
		}
		
		public static function options() {
			
			$email = get_option('admin_email');
			
			$ip_address = isset($_SERVER['SERVER_ADDR']) ? sanitize_text_field($_SERVER['SERVER_ADDR']) : '';
			
			$blackhole_options = array(
				
				'disable_login'    => true,
				'hits_required'    => 1,
				'email_alerts'     => true,
				'email_address'    => $email,
				'email_from'       => $email,
				'alert_type'       => 'default',
				'alert_message'    => '',
				'warning_message'  => 'default',
				'blocked_message'  => 'default',
				'blocked_custom'   => '<h1>'. esc_html__('You have been banned from this site.', 'blackhole-pro') .'</h1>',
				'blocked_redirect' => '',
				'trigger_format'   => 'default',
				'trigger_location' => 'footer',
				'trigger_custom'   => '',
				'trigger_keyword'  => 'blackhole',
				'excluded_pages'   => '',
				'disable_trigger'  => false,
				'botlog_items'     => 10,
				'bot_whitelist'    => self::default_uas(),
				'ip_whitelist'     => self::default_ips(),
				'status_code'      => '403',
				'bot_blacklist'    => '',
				
			);
			
			return apply_filters('blackhole_options', $blackhole_options);
			
		}
		
		public static function badbots() {
			
			$default = array(
				'bot_id'       => 0,
				'ip_address'   => '173.203.204.123',
				'request_uri'  => 'https://example.com/?blackhole=h1821643',
				'remote_host'  => 'ngc1600.galaxy.example.com',
				'user_agent'   => 'Cygnus X-1 (Space Invaders) User Agent (Atari 2600)',
				'referrer'     => 'https://domain.tld/',
				'protocol'     => 'HTTP/2.0',
				'method'       => 'GET',
				'date'         => '2050/07/04 @ 12:00:00 am',
				'code'         => 'US',
				'country'      => 'United States',
				'region'       => 'Texas',
				'city'         => 'San Antonio',
				'zip'          => '78218',
				'hits'         => 1,
				'latest'       => '2050/07/04 @ 12:00:00 am',
			);
			
			$default = apply_filters('blackhole_default_bot', $default);
			
			$badbots = array($default);
			
			return apply_filters('blackhole_badbots', $badbots);
			
		}
		
		private function constants() {
			
			if (!defined('BBB_REQUIRE')) define('BBB_REQUIRE', '4.6');
			if (!defined('BBB_VERSION')) define('BBB_VERSION', '3.4.1');
			if (!defined('BBB_NAME'))    define('BBB_NAME',    'Blackhole Pro');
			if (!defined('BBB_AUTHOR'))  define('BBB_AUTHOR',  'Jeff Starr');
			if (!defined('BBB_HOME'))    define('BBB_HOME',    'https://plugin-planet.com');
			if (!defined('BBB_URL'))     define('BBB_URL',     plugin_dir_url(__FILE__));
			if (!defined('BBB_DIR'))     define('BBB_DIR',     plugin_dir_path(__FILE__));
			if (!defined('BBB_FILE'))    define('BBB_FILE',    plugin_basename(__FILE__));
			if (!defined('BBB_SLUG'))    define('BBB_SLUG',    basename(dirname(__FILE__)));
			
		}
		
		private function includes() {
			
			require_once BBB_DIR .'inc/blackhole-alerts.php';
			require_once BBB_DIR .'inc/blackhole-core.php';
			require_once BBB_DIR .'inc/blackhole-geo.php';
			require_once BBB_DIR .'inc/blackhole-helpers.php';
			require_once BBB_DIR .'inc/blackhole-ip.php';
			require_once BBB_DIR .'inc/blackhole-response.php';
			require_once BBB_DIR .'inc/blackhole-trigger.php';
			
			require_once BBB_DIR .'inc/settings-register.php';
			require_once BBB_DIR .'inc/settings-reset.php';
			
			if (is_admin()) {
				
				require_once BBB_DIR .'inc/array-column.php';
				require_once BBB_DIR .'inc/status-codes.php';
				require_once BBB_DIR .'inc/contextual-help.php';
				require_once BBB_DIR .'inc/resources-enqueue.php';
				require_once BBB_DIR .'inc/settings-display.php';
				require_once BBB_DIR .'inc/badbots-display.php';
				require_once BBB_DIR .'updates/updates.php';
				
			}
		}
		
		public static function default_ips() {
			
			$ip_server = isset($_SERVER['SERVER_ADDR']) ? sanitize_text_field($_SERVER['SERVER_ADDR']) : '';
			$ip_remote = isset($_SERVER['REMOTE_ADDR']) ? sanitize_text_field($_SERVER['REMOTE_ADDR']) : '';
			
			$ips = $ip_server . "\n" . $ip_remote;
			
			return apply_filters('blackhole_default_ips', $ips);
			
		}
		
		public static function default_uas() {
			
			$uas  = 'a6-indexer, adsbot-google, ahrefsbot, aolbuild, apis-google, baidu, bingbot, bingpreview, butterfly, chrome, cloudflare, duckduckgo, embedly, ';
			$uas .= 'facebookexternalhit, facebot, googlebot, google page speed, ia_archiver, linkedinbot, mediapartners-google, msnbot, netcraftsurvey, ';
			$uas .= 'outbrain, pinterest, quora, rogerbot, showyoubot, slackbot, slurp, sogou, teoma, tweetmemebot, twitterbot, ';
			$uas .= 'uptimerobot, urlresolver, vkshare, w3c_validator, wordpress, wp rocket, yandex';
			
			return apply_filters('blackhole_default_uas', $uas);
			
		}
		
		public function import_data() {
			
			$update = get_option('blackhole_pro_update', 1);
			
			if (!$update) return;
			
			$date = blackhole_get_date();
			
			$bbb_options = get_option('bbb_options', array());
			$bbb_badbots = get_option('bbb_badbots', array());
			
			$blackhole_options = get_option('blackhole_pro_options', self::$instance->options());
			$blackhole_badbots = get_option('blackhole_pro_badbots', self::$instance->badbots());
			
			$bbb_wlist_ips = isset($bbb_options['ip_whitelist'])  ? trim($bbb_options['ip_whitelist'],  ', ') : '';
			$bbb_wlist_uas = isset($bbb_options['bot_whitelist']) ? trim($bbb_options['bot_whitelist'], ', ') : '';
			
			$blackhole_wlist_ips = isset($blackhole_options['ip_whitelist'])  ? trim($blackhole_options['ip_whitelist'],  ', ') : '';
			$blackhole_wlist_uas = isset($blackhole_options['bot_whitelist']) ? trim($blackhole_options['bot_whitelist'], ', ') : '';
			
			$whitelist_ips = $blackhole_wlist_ips . "\n" . $bbb_wlist_ips;
			$whitelist_uas = $blackhole_wlist_uas . ', ' . $bbb_wlist_uas;
			
			$explode_ips   = array_map('trim', explode("\n", $whitelist_ips));
			$explode_ips   = array_unique($explode_ips);
			$whitelist_ips = implode("\n", $explode_ips);
			
			$explode_uas   = array_map('trim', explode(',', $whitelist_uas));
			$explode_uas   = array_unique($explode_uas);
			$whitelist_uas = implode(', ', $explode_uas);
			
			if (isset($blackhole_options['ip_whitelist']))  $blackhole_options['ip_whitelist']  = $whitelist_ips;
			if (isset($blackhole_options['bot_whitelist'])) $blackhole_options['bot_whitelist'] = $whitelist_uas;
			
			$key = count($blackhole_badbots);
			
			$new = array();
			
			foreach ($bbb_badbots as $badbot) {
				
				extract($badbot); // $ip_address, $request_uri, $query_string, $user_agent, $referrer, $protocol, $method, $date
				
				if ($ip_address === '173.203.204.123') continue;
				
				$new[$key] = array(
					
					'bot_id'       => $key,
					'ip_address'   => $ip_address,
					'request_uri'  => $request_uri,
					'remote_host'  => '',
					'user_agent'   => $user_agent,
					'referrer'     => $referrer,
					'protocol'     => $protocol,
					'method'       => $method,
					'date'         => $date,
					'code'         => '',
					'country'      => esc_html__('Imported from free version', 'blackhole-pro'),
					'region'       => '',
					'city'         => '',
					'zip'          => '',
					'hits'         => 1,
					'latest'       => $date
				);
				
				$key++;
				
			}
			
			$blackhole_badbots = apply_filters('blackhole_import_bots', array_merge($blackhole_badbots, $new));
			
			$temp = array_unique(array_column($blackhole_badbots, 'ip_address'));
			
			$blackhole_badbots = array_intersect_key($blackhole_badbots, $temp);
			
			$update_options = update_option('blackhole_pro_options', $blackhole_options);
			$update_badbots = update_option('blackhole_pro_badbots', $blackhole_badbots);
			
			if ($update_options || $update_badbots) $update = update_option('blackhole_pro_update', 0);
			
			return $update;
			
		}
		
		public function action_links($links, $file) {
			if ($file == BBB_FILE && current_user_can('manage_options')) {
				
				$blackhole_links = '<a href="'. admin_url('admin.php?page=blackhole_settings') .'">'. esc_html__('Settings', 'blackhole-pro') .'</a>';
				array_unshift($links, $blackhole_links);
				
			}
			return $links;
		}
		
		public function plugin_links($links, $file) {
			if ($file == BBB_FILE) {
				
				$rate_href  = 'https://wordpress.org/support/plugin/blackhole-bad-bots/reviews/?rate=5#new-post';
				$rate_title = esc_attr__('Click here to rate and review this plugin on WordPress.org', 'blackhole-pro');
				$rate_text  = esc_html__('Rate this plugin&nbsp;&raquo;', 'blackhole-pro');
				
				$links[] = '<a target="_blank" rel="noopener noreferrer" href="'. $rate_href .'" title="'. $rate_title .'">'. $rate_text .'</a>';
				
			}
			return $links;
		}
		
		function footer_text($text) {
			
			$screen_id = blackhole_get_current_screen_id();
			
			$ids = array('toplevel_page_blackhole_settings', 'blackhole-pro_page_blackhole_badbots', 'blackhole-pro_page_blackhole_license');
			
			if ($screen_id && apply_filters('blackhole_admin_footer_text', in_array($screen_id, $ids))) {
				
				$text = __('Like this plugin? Give it a', 'blackhole-pro');
				
				$text .= ' <a target="_blank" rel="noopener noreferrer" href="https://wordpress.org/support/plugin/blackhole-bad-bots/reviews/?rate=5#new-post">';
				
				$text .= __('★★★★★ rating&nbsp;&raquo;', 'blackhole-pro') .'</a>';
				
			}
			
			return $text;
			
		}
		
		public function check_blackhole() {
			if (class_exists('Blackhole_Bad_Bots')) {
				if (is_plugin_active('blackhole-pro/blackhole-pro.php')) {
					deactivate_plugins(BBB_FILE);
					
					$msg  = '<strong>'. esc_html__('Warning:', 'blackhole-pro') .'</strong> '. esc_html__('Free and Pro versions of Blackhole cannot be activated at the same time. ', 'blackhole-pro');
					$msg .= esc_html__('Please return to the', 'blackhole-pro') .' <a href="'. admin_url('plugins.php') .'">'. esc_html__('WordPress Admin Area', 'blackhole-pro') .'</a> '. esc_html__('and try again.', 'blackhole-pro');
					
					wp_die($msg);
				}
			}
		}
		
		public function check_version() {
			$wp_version = get_bloginfo('version');
			if (isset($_GET['activate']) && $_GET['activate'] == 'true') {
				if (version_compare($wp_version, BBB_REQUIRE, '<')) {
					if (is_plugin_active(BBB_FILE)) {
						deactivate_plugins(BBB_FILE);
						$msg  = '<strong>'. BBB_NAME .'</strong> '. esc_html__('requires WordPress ', 'blackhole-pro') . BBB_REQUIRE . esc_html__(' or higher, and has been deactivated!', 'blackhole-pro') .'<br />';
						$msg .= esc_html__('Please return to the', 'blackhole-pro') .' <a href="'. admin_url() .'">'. esc_html__('WP Admin Area', 'blackhole-pro') .'</a> '. esc_html__('to upgrade WordPress and try again.', 'blackhole-pro');
						wp_die($msg);
					}
				}
			}
		}
		
		public function load_i18n() {
			
			$domain = 'blackhole-pro';
			
			$locale = apply_filters('blackhole_i18n_locale', get_locale(), $domain);
			
			$dir    = trailingslashit(WP_LANG_DIR);
			
			$file   = $domain .'-'. $locale .'.mo';
			
			$path_1 = $dir . $file;
			
			$path_2 = $dir . $domain .'/'. $file;
			
			$path_3 = $dir .'plugins/'. $file;
			
			$path_4 = $dir .'plugins/'. $domain .'/'. $file;
			
			$paths = array($path_1, $path_2, $path_3, $path_4);
			
			foreach ($paths as $path) {
				
				if ($loaded = load_textdomain($domain, $path)) {
					
					return $loaded;
					
				} else {
					
					return load_plugin_textdomain($domain, false, dirname(BBB_FILE) .'/languages/');
					
				}
				
			}
			
		}
		
		public function blackhole_allow_style($styles) {
			
			$styles[] = 'display';
			
		    return $styles;
			
		}
		
		public function __clone() {
			_doing_it_wrong(__FUNCTION__, esc_html__('Cheatin&rsquo; huh?', 'blackhole-pro'), BBB_VERSION);
		}
		
		public function __wakeup() {
			_doing_it_wrong(__FUNCTION__, esc_html__('Cheatin&rsquo; huh?', 'blackhole-pro'), BBB_VERSION);
		}
		
	}
}

if (class_exists('Blackhole_Pro')) {
	
	$blackhole_options = get_option('blackhole_pro_options', Blackhole_Pro::options());
	$blackhole_badbots = get_option('blackhole_pro_badbots', Blackhole_Pro::badbots());
	
	$blackhole_options = apply_filters('blackhole_get_options', $blackhole_options);
	$blackhole_badbots = apply_filters('blackhole_get_badbots', $blackhole_badbots);
	
	if (!function_exists('blackhole_bad_bots')) {
		
		function blackhole_bad_bots() {
			
			return Blackhole_Pro::instance();
		}
	}
	
	blackhole_bad_bots();
	
}
