<?php // Blackhole Pro - Bad Bots Display

if (!defined('ABSPATH')) exit;

function blackhole_display_badbots() {
	
	global $blackhole_options, $blackhole_badbots;
	
	$count = count($blackhole_badbots);
	
	$filters = blackhole_get_filters();
	
	$blackhole_badbots = blackhole_search_filter();
	
	$filter = blackhole_sort_filter();
	
	usort($blackhole_badbots, function($a, $b) use($filter) {
		
		$x = isset($a[$filter]) ? $a[$filter] : null;
		$y = isset($b[$filter]) ? $b[$filter] : null;
		
		return strnatcasecmp($x, $y);
		
	});
	
	$page = (isset($_GET['p']) && is_numeric($_GET['p']) && ($_GET['p'] > 0)) ? intval($_GET['p']) : 1;
	
	$results = isset($blackhole_options['botlog_items']) ? intval($blackhole_options['botlog_items']) : 10;
	
	$alerts = isset($blackhole_options['alert_type']) ? $blackhole_options['alert_type'] : 'default';
	
	$alerts = ($alerts === 'daily' || $alerts === 'weekly') ? true : false;

	$offset = ($page - 1) * $results;
	
	$total = count($blackhole_badbots);
	
	$pages = ceil($total / $results);
	
	$url = blackhole_get_results_url($filter, $page);
	
	$nonce_update_bot = wp_create_nonce('blackhole_update_bot');
	$nonce_add_bot    = wp_create_nonce('blackhole_add_bot');
	
	?>
	
	<div class="wrap">
		
		<h1 class="bbb-title"><?php echo BBB_NAME; ?> <span><?php echo BBB_VERSION; ?></span></h1>
		
		<?php echo blackhole_license_status(); ?>
		
		<h2><?php esc_html_e('Welcome to Blackhole..', 'blackhole-pro'); ?></h2>
		<p><?php esc_html_e('Bad Bots Log showing all blocked bots. Hit the Help tab for details.', 'blackhole-pro'); ?></p>
		
		<div class="bbb-nav">
			
			<?php echo blackhole_badbots_intro($count, $total, $page, $pages, $filter); ?>
			
			<?php if ($total > 0 || $count > 0) : ?>
			
			<select name="filter" onchange="blackhole_auto_submit('parent',this,0);">
				<?php
					
					foreach ($filters as $k => $v) {
						
						$value = array('filter' => $k);
						
						$value = add_query_arg($value, $url);
						
						$selected = ($k === $filter) ? ' selected="selected"' : '';
						
						echo '<option value="'. esc_url($value) .'"'. $selected .'>'. $v .'</option>';
						
					}
					
				?>
			</select>
		
			<select name="paging" onchange="blackhole_auto_submit('parent',this,0);">
				<?php 
					
					for ($i = 1; $i < ($pages + 1); ++$i) {
						
						$value = array('p' => $i);
						
						$value = add_query_arg($value, $url);
						
						$selected = ($i === $page) ? ' selected="selected"' : '';
						
						echo '<option value="'. esc_url($value) .'"'. $selected .'>'. esc_html__('Page ', 'blackhole-pro') . strval($i) .'</option>';
						
					}
					
				?>
			</select>
			
			<div class="bbb-nav-buttons">
				<?php 
					
					if ($page > 1) {
						
						$value = array('p' => $page - 1);
						
						$value = add_query_arg($value, $url);
						
						echo '<a class="button" href="'. esc_url($value) .'">&laquo; '. esc_html__('Previous', 'blackhole-pro') .'</a> ';
						
					}
					
					if ($page < $pages) {
						
						$value = array('p' => $page + 1);
						
						$value = add_query_arg($value, $url);
						
						echo '<a class="button" href="'. esc_url($value) .'">'. esc_html__('Next', 'blackhole-pro') .' &raquo;</a> ';
					}
					
				?>
			</div>
			
			<form class="bbb-nav-search" method="post">
				<input type="search" size="40" name="search-query" value="<?php echo blackhole_get_search_query(); ?>" placeholder="<?php esc_attr_e('Search', 'blackhole-pro'); ?>"> 
				<select name="search-filter">
					<?php
						
						foreach ($filters as $k => $v) {
							
							$selected = ($k === $filter) ? ' selected="selected"' : '';
							
							if ($k === 'bot_id') {
								
								$k = 'everything';
								$v = esc_html__('Everything', 'blackhole-pro');
								
							}
							
							$selected = ($k === blackhole_get_search_filter()) ? ' selected="selected"' : '';
							
							echo '<option value="'. $k .'"'. $selected .'>'. $v .'</option>';
							
						}
						
					?>
				</select>
				<input type="submit" class="button" value="<?php _e('Search', 'blackhole-pro'); ?>">
			</form>
			
			<?php endif; ?>
			
		</div>
		
		<div class="bbb-bots">
			
			<form method="post">
				<table class="widefat fixed striped">
					<thead>
						<?php blackhole_table_header(); ?>
					</thead>
					<tbody>
					<?php
						
						if (!empty($blackhole_badbots)) {
							
							for ($j = 0; $j < $total; ++$j) {
								
								if (($j >= intval($offset)) && ($j < (intval($offset) + intval($results)))) {
									
									foreach ($filters as $k => $v) {
										
										${$k} = isset($blackhole_badbots[$j][$k]) ? $blackhole_badbots[$j][$k] : '';
										
									}
									
									$c1 = blackhole_get_flag($code);
									$c2 = blackhole_get_location($bot_id, $ip_address, $country, $region, $city, $zip);
									$c3 = blackhole_get_meta($bot_id, $date, $hits, $latest, $ip_address);
									$c4 = blackhole_get_request($remote_host, $request_uri, $user_agent, $protocol, $referrer);
									
									?>
									
									<tr>
										<th class="c c0 check-column">
											<input type="checkbox" name="bot-id[]" value="<?php echo $bot_id; ?>" title="Bot ID: <?php echo $bot_id; ?>">
										</th>
										<td class="c c1"><?php echo $c1; ?></td>
										<td class="c c2"><?php echo $c2; ?></td>
										<td class="c c3"><?php echo $c3; ?></td>
										<td class="c c4<?php if ($alerts) : echo $reported ? ' r-'. strval($reported) : ' r-0'; endif; ?>"><?php echo $c4; ?></td>
									</tr>
									
									<?php
									
								}
								
							}
							
						} else {
							
							echo '<tr><td colspan="5">No bots found.</td></tr>';
							
						} 
					
					?>
					
					</tbody>
					<tfoot>
						<?php blackhole_table_header(); ?>
					</tfoot>
				</table>
				
				<input type="hidden" name="update-bots-verify" value="<?php echo esc_attr($nonce_update_bot); ?>">
				<input type="submit" name="delete-bots" value="<?php esc_attr_e('Delete', 'blackhole-pro'); ?>" class="button button-primary">
				<input type="submit" name="wlist-bots-ip" value="<?php esc_attr_e('Whitelist IP', 'blackhole-pro'); ?>" class="button button-primary">
				<input type="submit" name="wlist-bots-ua" value="<?php esc_attr_e('Whitelist User Agent', 'blackhole-pro'); ?>" class="button button-primary">
				
			</form>
			
		</div>
		
		<div class="bbb-add">
			
			<h2><?php esc_html_e('Add bot', 'blackhole-pro'); ?></h2>
			<p><?php esc_html_e('Add a bad bot to the log. IP Address required, everything else optional.', 'blackhole-pro'); ?></p>
			
			<form method="post">
				<table>
					<tr>
						<td><label for="ip_address"><?php esc_html_e('IP Address', 'blackhole-pro'); ?></label></td>
						<td><input class="regular-text" type="text" size="40" name="ip_address" id="ip_address" placeholder="<?php esc_html_e('173.203.204.123', 'blackhole-pro'); ?>"></td>
					</tr>
					<tr>
						<td><label for="request_uri"><?php esc_html_e('Request URI', 'blackhole-pro'); ?></label></td>
						<td><input class="regular-text" type="text" size="40" name="request_uri" id="request_uri" placeholder="<?php esc_html_e('/?blackhole=h1821643', 'blackhole-pro'); ?>"></td>
					</tr>
					<tr>
						<td><label for="remote_host"><?php esc_html_e('Remote Host', 'blackhole-pro'); ?></label></td>
						<td><input class="regular-text" type="text" size="40" name="remote_host" id="remote_host" placeholder="<?php esc_html_e('ngc1600.galaxy.example.com', 'blackhole-pro'); ?>"></td>
					</tr>
					<tr>
						<td><label for="user_agent"><?php esc_html_e('User Agent', 'blackhole-pro'); ?></label></td>
						<td><input class="regular-text" type="text" size="40" name="user_agent" id="user_agent" placeholder="<?php esc_html_e('Cygnus X-1 (Space Invaders)', 'blackhole-pro'); ?>"></td>
					</tr>
					<tr>
						<td><label for="referrer"><?php esc_html_e('Referrer', 'blackhole-pro'); ?></label></td>
						<td><input class="regular-text" type="text" size="40" name="referrer" id="referrer" placeholder="<?php esc_html_e('https://domain.tld/', 'blackhole-pro'); ?>"></td>
					</tr>
					<tr>
						<td><label for="protocol"><?php esc_html_e('Protocol', 'blackhole-pro'); ?></label></td>
						<td><input class="regular-text" type="text" size="40" name="protocol" id="protocol" placeholder="<?php esc_html_e('HTTP/2.0', 'blackhole-pro'); ?>"></td>
					</tr>
				</table>
				
				<input type="hidden" name="add-bots-verify" value="<?php echo esc_attr($nonce_add_bot); ?>">
				<input type="submit" name="add-bots" value="<?php esc_attr_e('Add bot', 'blackhole-pro'); ?>" class="button button-primary">
			</form>
			
		</div>
		
	</div>
	
	<?php
	
}



function blackhole_search_filter() {
	
	global $blackhole_badbots;
	
	$search_query = blackhole_get_search_query();
	$search_filter = blackhole_get_search_filter();
	
	if (!empty($search_query)) {
		
		$search_results = array();
		
		foreach ($blackhole_badbots as $key => $value) {
			
			foreach ($value as $k => $v) {
				
				if (stripos($v, $search_query) !== false) {
					
					if (($search_filter === 'everything') || (stripos($k, $search_filter) !== false)) {
						
						$search_results[$key] = $key;
						
					}
					
				}
				
			}
			
		}
		
		$search_results = array_unique($search_results);
		$blackhole_badbots = array_intersect_key($blackhole_badbots, $search_results);
		$blackhole_badbots = array_values($blackhole_badbots);
		
	}
	
	return apply_filters('blackhole_log_data', $blackhole_badbots);
	
}

function blackhole_sort_filter() {
	
	$filter = isset($_GET['filter']) ? $_GET['filter'] : 'bot_id';
	
	$filter = preg_match('/^[a-zA-Z0-9_]+$/', $filter) ? $filter : 'bot_id';
	
	return $filter;
	
}

function blackhole_get_results_url($filter, $page) {
	
	$params = array('filter' => $filter, 'p' => $page);
	
	if (empty($params['filter'])) unset($params['filter']);
	
	if (empty($params['p'])) unset($params['p']);
	
	$path = admin_url('admin.php?page=blackhole_badbots');
	
	$url = add_query_arg($params, $path);
	
	return $url;
	
}

function blackhole_badbots_intro($count, $total, $page, $pages, $filter) {
	
	$search_query = blackhole_get_search_query();
	$search_filter = blackhole_get_search_filter();
	
	$line1 = $line2 = $line3 = $line4 = $line5 = '';
	
	if (isset($search_query) && $search_query !== false && $search_query !== '') {
		
	 	$line1 = '<span>'. esc_html__('Search: ', 'blackhole-pro') . $search_query .'</span> ';
		
		if (!empty($search_filter)) {
			
			$line2 = '<span>'. esc_html__('Searching: ', 'blackhole-pro') . blackhole_uppercase($search_filter) .'</span> ';
			
		}
	}
	
	if ($total > 0) {
		
		$line3 = esc_html__(' of ', 'blackhole-pro') . $count .'</span> ';
		$line4 = '<span>'. esc_html__('Page: ', 'blackhole-pro') . strval($page) . esc_html__(' of ', 'blackhole-pro') . strval($pages) .'</span> ';
		$line5 = '<span>'. esc_html__('Sort: ', 'blackhole-pro') . blackhole_uppercase($filter) .'</span>';
		
	}
	
	$intro  = '<div class="intro">'. $line1 . $line2;
	$intro .= '<span>'. esc_html__('Results: ', 'blackhole-pro') . $total . $line3;
	$intro .= $line4 . $line5 .'</div>';
	
	return $intro;
	
}

function blackhole_uppercase($string) {
	
	return ucwords(str_replace('uri', 'URI', str_replace('id', 'ID', str_replace('ip', 'IP', str_replace('_', ' ', $string)))));
	
}

function blackhole_get_search_query() {
	
	return isset($_POST['search-query']) ? $_POST['search-query'] : false;
	
}

function blackhole_get_search_filter() {
	
	return isset($_POST['search-filter']) ? $_POST['search-filter'] : false;
	
}

function blackhole_table_header() { ?>
	
	<tr>
		<td class="c0 check-column"><input type="checkbox" title="Select All"></td>
		<th class="c1"><?php esc_html_e('Flag',    'blackhole-pro'); ?></th>
		<th class="c2"><?php esc_html_e('Geo',     'blackhole-pro'); ?></th>
		<th class="c3"><?php esc_html_e('Meta',    'blackhole-pro'); ?></th>
		<th class="c4"><?php esc_html_e('Request', 'blackhole-pro'); ?></th>
	</tr>
	
<?php }

function blackhole_get_filters() {
	
	return array(
		'bot_id'      => esc_html__('Sort By:',     'blackhole-pro'), 
		'ip_address'  => esc_html__('IP Address',   'blackhole-pro'), 
		'request_uri' => esc_html__('Request URI',  'blackhole-pro'), 
		'remote_host' => esc_html__('Remote Host',  'blackhole-pro'), 
		'user_agent'  => esc_html__('User Agent',   'blackhole-pro'), 
		'referrer'    => esc_html__('Referrer',     'blackhole-pro'), 
		'protocol'    => esc_html__('Protocol',     'blackhole-pro'), 
		'date'        => esc_html__('Date',         'blackhole-pro'),
		'code'        => esc_html__('Code',         'blackhole-pro'),
		'country'     => esc_html__('Country',      'blackhole-pro'),
		'region'      => esc_html__('Region',       'blackhole-pro'),
		'city'        => esc_html__('City',         'blackhole-pro'),
		'zip'         => esc_html__('Zip',          'blackhole-pro'),
		'hits'        => esc_html__('Hits',         'blackhole-pro'),
		'latest'      => esc_html__('Latest Hit',   'blackhole-pro'),
		'reported'    => esc_html__('Reported',     'blackhole-pro'),
	);
	
}

function blackhole_get_flag($code) {
	
	$file = file_exists(BBB_DIR .'img/flags/'. strtolower($code) .'.svg') ? strtolower($code) : 'anon';
	
	$url = BBB_URL .'img/flags/'. $file .'.svg';
	
	$flag = '<img src="'. $url .'" width="60" alt="'. $code .'">';
	
	return $flag;
}

function blackhole_get_location($bot_id, $ip_address, $country, $region, $city, $zip) {
	
	$ip_address = empty($ip_address) ? '[n/a]' : $ip_address;
	$country    = empty($country)    ? '[n/a]' : str_replace(' ', '&nbsp;', $country);
	$region     = empty($region)     ? '[n/a]' : str_replace(' ', '&nbsp;', $region);
	$city       = empty($city)       ? '[n/a]' : str_replace(' ', '&nbsp;', $city);
	$zip        = empty($zip)        ? '[n/a]' : str_replace(' ', '&nbsp;', $zip);
	
	$lookup_href = 'https://whatismyipaddress.com/ip/'. $ip_address;
	$lookup_title = esc_html__('Whois lookup (via IP address)', 'blackhole-pro');
	
	$loc  = '<span><a class="bbb-ip-address" target="_blank" rel="noopener noreferrer" href="'. $lookup_href .'" title="'. $lookup_title .'">'. $ip_address .'</a></span> ';
	$loc .= '<span>'. $country .', '. $region .', '. $city .', '. $zip .'</span>';
	$loc .= '<span>'. blackhole_get_geo_link($bot_id, $ip_address) .'</span>';
	
	return $loc;
	
}

function blackhole_get_geo_link($bot_id, $ip_address) {
	
	$nonce = wp_create_nonce('blackhole_do_geo_lookup');
	
	$args = array('blackhole-geo-lookup' => $nonce, 'blackhole-bot-id' => $bot_id, 'blackhole-ip-address' => $ip_address);
	
	$href = esc_url(add_query_arg($args, admin_url('admin.php?page=blackhole_badbots')));
	
	$text = esc_html__('Refresh Geo', 'blackhole-pro');
	
	return '<a class="bbb-geo-lookup" href="'. $href .'">'. $text .'</a>';
	
}

function blackhole_get_meta($bot_id, $date, $hits, $latest, $ip_address) {
	
	$lookup_href = 'https://www.whois.com/whois/'. $ip_address;
	
	$lookup_title = esc_html__('Whois lookup (forward/reverse)', 'blackhole-pro');
	
	$meta  = '<span>'. esc_html__('Bot ID:', 'blackhole-pro');
	$meta .= ' <a target="_blank" rel="noopener noreferrer" href="'. $lookup_href .'" title="'. $lookup_title .'">'. $bot_id .'</a></span> ';
	$meta .= '<span>'. esc_html__('Blocked Hits: ', 'blackhole-pro') . $hits .'</span> ';
	$meta .= '<span class="tip" title="'. esc_html__('Latest Hit: ', 'blackhole-pro') . $latest .'">'. $date .'</span>';
	
	return $meta;
	
}

function blackhole_get_request($remote_host, $request_uri, $user_agent, $protocol, $referrer) {
	
	$remote_host = empty($remote_host) ? '[n/a]' : $remote_host;
	$request_uri = empty($request_uri) ? '[n/a]' : $request_uri;
	$user_agent  = empty($user_agent)  ? '[n/a]' : $user_agent;
	$protocol    = empty($protocol)    ? '[n/a]' : $protocol;
	
	$lookup_href = 'https://dnsquery.org/dnsquery/'. $remote_host .'/ANY';
	$lookup_title = esc_html__('Whois lookup (via hostname)', 'blackhole-pro');
	
	$info  = '<span>'. esc_html__('Host:', 'blackhole-pro');
	$info .= ' <a target="_blank" rel="noopener noreferrer" href="'. $lookup_href .'" title="'. $lookup_title .'">'. $remote_host .'</a></span> ';
	$info .= '<span class="tip" title="'. esc_html__('Protocol: ', 'blackhole-pro') . $protocol .'">';
	$info .= esc_html__('Request: ', 'blackhole-pro') . $request_uri .'</span> ';
	$info .= $referrer ? '<span>'. esc_html__('Referrer: ', 'blackhole-pro') . $referrer .'</span>' : '';
	$info .= '<span>'. esc_html__('User Agent: ', 'blackhole-pro') . $user_agent .'</span>';
	
	return $info;
	
}
