<?php // Blackhole Pro - Email Alerts

if (!defined('ABSPATH')) exit;

function blackhole_cron_deactivation() {
	
	wp_clear_scheduled_hook('blackhole_send_email_cron');
	
}

function blackhole_send_email_cron() {
	
	if (!defined('DOING_CRON')) return;
	
	global $blackhole_options;
	
	if (isset($blackhole_options['email_alerts']) && !$blackhole_options['email_alerts']) return false;
	
	$type = wp_get_schedule('blackhole_send_email_cron');
	
	$name = apply_filters('blackhole_alert_name', get_option('blogname'));
	
	$domain = parse_url(get_home_url(), PHP_URL_HOST);
	
	$address = blackhole_get_email_address();
	
	$admin_email = get_bloginfo('admin_email');
	
	$login = get_dashboard_url();
	
	//
	
	$subject = blackhole_alert_subject($name, $domain, $type);
	
	$message = blackhole_alert_message(array(), '', $name, $domain, $type, $login); // todo: $whois
	
	if (isset($address[0]) && !empty($address[0])) {
		
		foreach ($address as $k => $v) {
			
			$email = (isset($v['email']) && !empty($v['email'])) ? $v['email'] : $admin_email;
			$from  = (isset($v['from'])  && !empty($v['from']))  ? $v['from']  : $admin_email;
			
			$headers = blackhole_email_headers($name, $from);
			
			wp_mail($email, $subject, $message, $headers);
			
		}
		
	}
	
}

function blackhole_send_email($vars, $whois) {
	
	global $blackhole_options;
	
	if (isset($blackhole_options['email_alerts']) && !$blackhole_options['email_alerts']) return false;
	
	$type = isset($blackhole_options['alert_type']) ? $blackhole_options['alert_type'] : 'default';
	
	if ($type === 'daily' || $type === 'weekly') return false;
	
	$name = apply_filters('blackhole_alert_name', get_option('blogname'));
	
	$domain = parse_url(get_home_url(), PHP_URL_HOST);
	
	$address = blackhole_get_email_address();
	
	$admin_email = get_bloginfo('admin_email');
	
	$login = get_dashboard_url();
	
	//
	
	$subject = blackhole_alert_subject($name, $domain, $type);
	
	$message = blackhole_alert_message($vars, $whois, $name, $domain, $type, $login);
	
	if (isset($address[0]) && !empty($address[0])) {
		
		foreach ($address as $k => $v) {
			
			$email = (isset($v['email']) && !empty($v['email'])) ? $v['email'] : $admin_email;
			$from  = (isset($v['from'])  && !empty($v['from']))  ? $v['from']  : $admin_email;
			
			$headers = blackhole_email_headers($name, $from);
			
			wp_mail($email, $subject, $message, $headers);
			
		}
		
	}
		
}

function blackhole_get_email_address() {
	
	global $blackhole_options;
	
	$email = (isset($blackhole_options['email_address']) && !empty($blackhole_options['email_address'])) ? $blackhole_options['email_address'] : get_option('admin_email');
	
	$from  = (isset($blackhole_options['email_from'])    && !empty($blackhole_options['email_from']))    ? $blackhole_options['email_from'] : $email;
	
	$email = explode(',', $email);
	
	$from  = explode(',', $from);
	
	$address = array();
	
	foreach ($email as $k => $v) $address[$k]['email'] = sanitize_email($v);
	
	foreach ($from as $k => $v) $address[$k]['from'] = sanitize_email($v);
	
	return apply_filters('blackhole_email_address', $address);
	
}

function blackhole_alert_subject($name, $domain, $type) {
	
	if ($type === 'daily') {
		
		$pre = __('Daily ', 'blackhole-pro');
		
	} elseif ($type === 'weekly') {
		
		$pre = __('Weekly ', 'blackhole-pro');
		
	} else {
		
		$pre = '';
		
	}
	
	$subject = $pre . __('Bad Bot Report from', 'blackhole-pro') .' '. $name;
	
	return apply_filters('blackhole_alert_subject', $subject, $name, $domain, $type);
	
}

function blackhole_alert_message($vars, $whois, $name, $domain, $type, $login) {
	
	if ($type === 'custom') {
		
		$message = blackhole_alert_custom($vars, $whois, $name, $domain, $login);
		
	} elseif ($type === 'simple') {
		
		$message = blackhole_alert_simple($vars, $whois, $name, $domain, $login);
		
	} elseif ($type === 'daily' || $type === 'weekly') {
		
		$message = blackhole_alert_cron($name, $domain, $type); // todo: $whois
		
	} else {
		
		$message = blackhole_alert_default($vars, $whois, $name, $domain, $login);
		
	}
	
	return apply_filters('blackhole_alert_message', $message, $vars, $whois, $name, $domain, $type, $login);
	
}

function blackhole_email_headers($name, $from) {
	
	$headers  = 'X-Mailer: Blackhole Pro'. "\n";
	$headers .= 'From: '. $name .' <'. $from .'>'. "\n";
	$headers .= 'Content-Type: text/plain; charset='. get_option('blog_charset', 'UTF-8') . "\n";
	
	return apply_filters('blackhole_alert_headers', $headers);
	
}

function blackhole_alert_cron($name, $domain, $type) {
	
	global $blackhole_badbots;
	
	$logged_bots = '';
	
	$i = 0;
	
	$count = strval($i);
	
	foreach ($blackhole_badbots as $key => $value) {
		
		$reported = isset($value['reported']) ? true : false;
		
		if (!$reported) {
			
			$blackhole_badbots[$key]['reported'] = true;
			
			$bot_id      = isset($value['bot_id'])      ? $value['bot_id']      : 0;
			$date        = isset($value['date'])        ? $value['date']        : '';
			$hits        = isset($value['hits'])        ? $value['hits']        : '';
			$ip_address  = isset($value['ip_address'])  ? $value['ip_address']  : '';
			$remote_host = isset($value['remote_host']) ? $value['remote_host'] : '';
			$request_uri = isset($value['request_uri']) ? $value['request_uri'] : '';
			$user_agent  = isset($value['user_agent'])  ? $value['user_agent']  : '';
			
			$user_agent = blackhole_truncate($user_agent, 70);
			
			$ip_lookup  = 'https://www.whois.com/whois/'. $ip_address;
			
			$ip_lookup  = apply_filters('blackhole_alert_ip_lookup', $ip_lookup, $ip_address);
			
			$logged_bots .= __('Bot ID:', 'blackhole-pro') .' '. $bot_id .' - '. $date ."\n\n";
			
			$logged_bots .= __('Blocked Hits:',  'blackhole-pro') .' '. $hits . "\n";
			$logged_bots .= __('IP Address:',    'blackhole-pro') .' '. $ip_address . "\n";
			$logged_bots .= __('Host Name:',     'blackhole-pro') .' '. $remote_host . "\n";
			$logged_bots .= __('Request URI:',   'blackhole-pro') .' '. $request_uri . "\n"; 
			$logged_bots .= __('User Agent:',    'blackhole-pro') .' '. $user_agent . "\n";
			$logged_bots .= __('IP/Geo Lookup:', 'blackhole-pro') .' '. $ip_lookup . "\n\n\n";
			
			$i++;
			
		}
		
	}
	
	if ($i) {
		
		$update = update_option('blackhole_pro_badbots', $blackhole_badbots, true);
		
		$count = strval($i);
		
	}
	
	$message = __('This', 'blackhole-pro') .' '. $type .' '. __('report is sent from Blackhole Pro via your site,', 'blackhole-pro') .' '. $name .' @ '. $domain . "\n\n";
	
	if (!empty($logged_bots)) {
		
		$timeframe = ($type === 'daily') ? __('24 hours.', 'blackhole-pro') : __('7 days.', 'blackhole-pro');
		
		$message .= __('The following', 'blackhole-pro') .' '. $count .' '. __('bots fell into the blackhole trap during the past', 'blackhole-pro') .' '. $timeframe . "\n\n\n" . $logged_bots;
		
	} else {
		
		$message .= "\n" . __('Congrats! No new bots were trapped today :)', 'blackhole-pro') . "\n\n\n";
		
	}
	
	$message .= __('For more details, or to disable these reports, visit Blackhole Pro > Bad Bots. Here is a direct link (requires login):', 'blackhole-pro') .' '. admin_url('admin.php?page=blackhole_badbots') . "\n\n";
	$message .= __('Visit Blackhole Pro @ Plugin Planet:', 'blackhole-pro') .' https://plugin-planet.com/blackhole-pro/'. "\n\n";
	$message .= __('Thank you for using Blackhole Pro.', 'blackhole-pro');
	
	return apply_filters('blackhole_alert_cron', $message, $name, $domain, $type, $logged_bots, $i);
	
}

function blackhole_alert_default($vars, $whois, $name, $domain, $login) {
	
	$date        = isset($vars['date'])        ? $vars['date']        : '';
	$request_uri = isset($vars['request_uri']) ? $vars['request_uri'] : '';
	$ip_address  = isset($vars['ip_address'])  ? $vars['ip_address']  : '';
	$remote_host = isset($vars['remote_host']) ? $vars['remote_host'] : '';
	$user_agent  = isset($vars['user_agent'])  ? $vars['user_agent']  : '';
	
	$ip_lookup  = 'https://www.whois.com/whois/'. $ip_address;
	
	$ip_lookup = apply_filters('blackhole_alert_ip_lookup', $ip_lookup, $ip_address);
	
	$intro  = __('Hello! This email alert is sent from your WordPress site, ', 'blackhole-pro') . $name .' @ '. $domain .'. ';
	$intro .= __('There, you are using a plugin called Blackhole Pro. ', 'blackhole-pro');
	$intro .= __('This email alert tells you that the plugin is working great, ', 'blackhole-pro');
	$intro .= __('doing its job blocking bad bots. ', 'blackhole-pro');
	$intro .= __('Below you will find details about the bad bot that was denied access. ', 'blackhole-pro');
	$intro .= __('To disable these email alerts at any time, visit the plugin settings. ', 'blackhole-pro') . "\n\n";
	$intro .= __('Visit site: ', 'blackhole-pro') . $login;
	
	$message  = $intro . "\n\n";
	$message .= $date  . "\n\n";
	$message .= __('Request URI: ',   'blackhole-pro') . $request_uri        . "\n";
	$message .= __('IP Address: ',    'blackhole-pro') . $ip_address         . "\n";
	$message .= __('Host Name: ',     'blackhole-pro') . $remote_host        . "\n";
	$message .= __('User Agent: ',    'blackhole-pro') . $user_agent         . "\n\n";
	$message .= __('IP/Geo Lookup: ', 'blackhole-pro') . "\n\n" . $ip_lookup . "\n\n";
	$message .= __('Whois Lookup: ',  'blackhole-pro') . "\n\n" . $whois     . "\n\n";
	
	return apply_filters('blackhole_alert_default', $message, $vars, $whois, $name, $domain, $login);
	
}

function blackhole_alert_simple($vars, $whois, $name, $domain, $login) {
	
	$date        = isset($vars['date'])        ? $vars['date']        : '';
	$request_uri = isset($vars['request_uri']) ? $vars['request_uri'] : '';
	$ip_address  = isset($vars['ip_address'])  ? $vars['ip_address']  : '';
	$remote_host = isset($vars['remote_host']) ? $vars['remote_host'] : '';
	$user_agent  = isset($vars['user_agent'])  ? $vars['user_agent']  : '';
	
	$ip_lookup = 'https://www.whois.com/whois/'. $ip_address;
	
	$ip_lookup = apply_filters('blackhole_alert_ip_lookup', $ip_lookup, $ip_address);
	
	$message  = $date . "\n\n";
	$message .= __('Website: ',       'blackhole-pro') . $name .' @ '. $domain . "\n\n";
	$message .= __('Request URI: ',   'blackhole-pro') . $request_uri . "\n\n";
	$message .= __('IP Address: ',    'blackhole-pro') . $ip_address  . "\n\n";
	$message .= __('Host Name: ',     'blackhole-pro') . $remote_host . "\n\n";
	$message .= __('User Agent: ',    'blackhole-pro') . $user_agent  . "\n\n";
	$message .= __('IP/Geo Lookup: ', 'blackhole-pro') . $ip_lookup   . "\n\n";
	
	$message .= __('Visit site: ', 'blackhole-pro') . $login . "\n\n";
	
	return apply_filters('blackhole_alert_simple', $message, $vars, $whois, $name, $domain, $login);
	
}

function blackhole_alert_custom($vars, $whois, $name, $domain, $login) {
	
	global $blackhole_options;
	
	$message = isset($blackhole_options['alert_message']) ? $blackhole_options['alert_message'] : '';
	
	$current_user = wp_get_current_user();
	$user = $current_user->user_login;
	$user = empty($user) ? '[n/a]' : $user;
	
	foreach ($vars as $k => $v) ${$k} = isset($vars[$k]) ? $vars[$k] : '[n/a]';
	
	$ip_lookup  = 'https://www.whois.com/whois/'. $ip_address;
	
	$ip_lookup = apply_filters('blackhole_alert_ip_lookup', $ip_lookup, $ip_address);
	
	$p = array();
	$p[0]  = "/\[ip_address\]/";
	$p[1]  = "/\[request_uri\]/";
	$p[2]  = "/\[remote_host\]/";
	$p[3]  = "/\[user_agent\]/";
	$p[4]  = "/\[referrer\]/";
	$p[5]  = "/\[protocol\]/";
	$p[6]  = "/\[method\]/";
	$p[7]  = "/\[date\]/";
	$p[8]  = "/\[whois\]/";
	$p[9]  = "/\[user\]/";
	$p[10] = "/\[code\]/";
	$p[11] = "/\[country\]/";
	$p[12] = "/\[region\]/";
	$p[13] = "/\[city\]/";
	$p[14] = "/\[zip\]/";
	$p[15] = "/\[hits\]/";
	$p[16] = "/\[latest\]/";
	$p[17] = "/\[site_name\]/";
	$p[18] = "/\[domain\]/";
	$p[19] = "/\[ip_lookup\]/";
	$p[20] = "/\[login\]/";
	
	$r = array();
	$r[0]  = $ip_address;
	$r[1]  = $request_uri;
	$r[2]  = $remote_host;
	$r[3]  = $user_agent;
	$r[4]  = $referrer;
	$r[5]  = $protocol;
	$r[6]  = $method;
	$r[7]  = $date;
	$r[8]  = $whois;
	$r[9]  = $user;
	$r[10] = $code;
	$r[11] = $country;
	$r[12] = $region;
	$r[13] = $city;
	$r[14] = $zip;
	$r[15] = $hits;
	$r[16] = $latest;
	$r[17] = $name;
	$r[18] = $domain;
	$r[19] = $ip_lookup;
	$r[20] = $login;
	
	$message = preg_replace($p, $r, $message);
	
	return apply_filters('blackhole_alert_custom', $message, $vars, $whois, $name, $domain, $login);
	
}