<?php // Blackhole Pro - Blackhole Core

if (!defined('ABSPATH')) exit;

function blackhole_robots_rules($path) {
	
	global $blackhole_options;
	
	$keyword = isset($blackhole_options['trigger_keyword']) ? $blackhole_options['trigger_keyword'] : 'blackhole';
	
	$rules = "\n" .'User-agent: *'. "\n" .'Disallow: /*'. $keyword . "\n" .'Disallow: /?'. $keyword . "\n";
	
	return apply_filters('blackhole_robots_rules', $rules, $path);
	
}

function blackhole_robots_wordpress($output, $public) {
	
	return $output . blackhole_robots();
	
}

function blackhole_domain() {
	
	$server_name = isset($_SERVER['SERVER_NAME']) ? htmlentities($_SERVER['SERVER_NAME'], ENT_QUOTES) : '';
	$http_host   = isset($_SERVER['HTTP_HOST'])   ? htmlentities($_SERVER['HTTP_HOST'],   ENT_QUOTES) : $server_name;
	
	return apply_filters('blackhole_domain', $http_host, $server_name);
	
}

function blackhole_robots() {
	
	$protocol = is_ssl() ? 'https://' : 'http://';
	
	$base_url = esc_url($protocol . blackhole_domain());
	
	$site_url = site_url();
	
	$path = str_replace($base_url, '', $site_url);
	
	$check = strpos($site_url, $base_url);
	
	if (($check === false) || (filter_var($site_url, FILTER_VALIDATE_URL) === false) || (filter_var($base_url, FILTER_VALIDATE_URL) === false)) {
		
		$robots = '';
		
	} else {
		
		$robots = blackhole_robots_rules($path);
		
	}
	
	return $robots;
	
}

function blackhole_scanner() {
	
	global $blackhole_options;
	
	$vars = blackhole_get_vars();
	
	if (blackhole_abort($vars)) return false;
	
	if (blackhole_blacklist($vars)) blackhole_display_message();
	
	blackhole_redirect_whitelist($vars);
	
	foreach ($vars as $k => $v) ${$k} = isset($vars[$k]) ? $vars[$k] : '';
	
	$keyword = isset($blackhole_options['trigger_keyword']) ? $blackhole_options['trigger_keyword'] : 'blackhole';
	$allowed = isset($blackhole_options['hits_required']) ? intval($blackhole_options['hits_required']) : 1;
	
	$hits = blackhole_get_hits($ip_address);
	$hits = isset($hits['hits']) ? intval($hits['hits']) : 1;
	
	$verify = isset($_GET[$keyword]) ? wp_verify_nonce($_GET[$keyword], 'blackhole_trigger') : false;
	$verify = apply_filters('blackhole_verify_nonce', $verify);
	
	if ($verify && isset($_GET[$keyword])) {
		
		blackhole_disable_cache();
		
		if ($hits >= $allowed) {
			
			if (blackhole_check_log($vars)) blackhole_display_message();
			else blackhole_display_warning($vars);
			
		} else {
			
			blackhole_display_warning($vars, false);
			
		}
		
	} else {
		
		if ($hits >= $allowed) {
				
			if (blackhole_check_log($vars)) {
				
				blackhole_disable_cache();
				
				blackhole_display_message();
				
			}
			
		}
		
	}
	
	do_action('blackhole_scanner', $vars, $hits);
	
	return false;
	
}

function blackhole_check_log($vars) {
	
	global $blackhole_badbots;
	
	if (!isset($blackhole_badbots) || empty($blackhole_badbots)) return false;
	
	foreach ($vars as $k => $v) ${$k} = isset($vars[$k]) ? $vars[$k] : '';
	
	$needle = apply_filters('blackhole_needle', 'ip_address', $vars, $blackhole_badbots);
	
	if (!isset(${$needle}) || empty(${$needle})) return false;
	
	if (!empty($ip)) $ip_address = $ip;
	
	foreach ($blackhole_badbots as $key => $value) {
		
		$value = (array) $value;
		
		$haystack = isset($value[$needle]) ? $value[$needle] : '';
		
		$find = stripos($haystack, ${$needle});
		
		if ($find !== false) {
			
			$blackhole_badbots[$key]['hits']   = isset($blackhole_badbots[$key]['hits']) ? intval($blackhole_badbots[$key]['hits']) + 1 : 1;
			$blackhole_badbots[$key]['latest'] = isset($blackhole_badbots[$key]['latest']) ? $date : $date;
			
			$update = update_option('blackhole_pro_badbots', $blackhole_badbots, true);
			
			return true;
			
		}
		
	}
	
	return false;
	
}

function blackhole_display_warning($vars, $log = true) {
	
	blackhole_get_deps();
	
	foreach ($vars as $k => $v) ${$k} = isset($vars[$k]) ? $vars[$k] : '';
	
	$whois = blackhole_whois($ip_address);
	
	blackhole_template($vars, $whois);
	
	if ($log) blackhole_log_bot($vars, $whois);
	
	exit;
	
}

function blackhole_log_bot($vars, $whois) {
	
	global $blackhole_badbots;
	
	$bot_id = blackhole_get_bot_id();
	
	foreach ($vars as $k => $v) ${$k} = isset($vars[$k]) ? $vars[$k] : '';
	
	$geo = blackhole_get_geo($ip_address);
	
	list ($code, $country, $region, $city, $zip) = $geo;
	
	$hits = 1;
	
	$vars = array(
		'bot_id'      => $bot_id,
		'ip_address'  => $ip_address, 
		'request_uri' => $request_uri, 
		'remote_host' => $remote_host, 
		'user_agent'  => $user_agent, 
		'referrer'    => $referrer, 
		'protocol'    => $protocol, 
		'method'      => $method, 
		'date'        => $date, 
		'code'        => $code,
		'country'     => $country,
		'region'      => $region,
		'city'        => $city,
		'zip'         => $zip,
		'hits'        => $hits, 
		'latest'      => $date,
	);
	
	$log = array($vars);
	
	$log = apply_filters('blackhole_log', $log, $vars);
	
	do_action('blackhole_log_bot', $log, $vars);
	
	$blackhole_badbots = array_merge($blackhole_badbots, $log);
	
	$update = update_option('blackhole_pro_badbots', $blackhole_badbots, true);
	
	$update = $update ? blackhole_send_email($vars, $whois) : false;

	return $update;
	
}