<?php // Blackhole Pro - Geo IP Lookups

if (!defined('ABSPATH')) exit;

function blackhole_get_geo($ip, $refresh = false) {
	
	$id = '12lQj0zvx17df4St9u1x';
	
	$lookup = 'https://zen-wp.com/api/?id='. $id .'&ip='. $ip .'&to=blackhole';
	
	$lookup = apply_filters('blackhole_geo_url', $lookup, $ip, $id);
	
	$error = '';
	
	if (function_exists('curl_version')) {
		
		$curl_connect = $refresh ? 4 : 2;
		$curl_timeout = $refresh ? 8 : 4;
		
		$curl = curl_init();
		
		curl_setopt($curl, CURLOPT_URL, $lookup);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_FAILONERROR, true);
		
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, $curl_connect);
		curl_setopt($curl, CURLOPT_TIMEOUT,        $curl_timeout);
		
		// curl_setopt($curl, CURLOPT_CONNECTTIMEOUT_MS, 500);
		// curl_setopt($curl, CURLOPT_TIMEOUT_MS, 700);
		
		curl_setopt($curl, CURLOPT_NOSIGNAL, 1);
		
		$json = curl_exec($curl);
		
		if (curl_errno($curl)) $error = __('Blackhole Pro: curl error: ', 'blackhole-pro') .' '. curl_error($curl);
		
		curl_close($curl);
		
	} else if (file_get_contents(__FILE__) && ini_get('allow_url_fopen')) {
		
		$ctx = stream_context_create(array('http' => array('header' => 'Blackhole Pro, ID: '. $id, 'timeout' => 1)));
		
		$json = @file_get_contents($lookup, 0, $ctx);
		
		if ($json === false) $error = __('Blackhole Pro: file_get_contents failed.', 'blackhole-pro');
		
	} else {
		
		$error = __('Blackhole Pro: Geo Lookup failed.', 'blackhole-pro');
		
	}
	
	if ($error) {
		
		$geo = array();
		
		$log = apply_filters('blackhole_pro_geo_lookup_log', 'GeoIP Lookup Error: '. $error, $error);
		
		// error_log($log);
		
	} else {
		
		$geo = json_decode($json, true);
		
	}
	
	$code    = isset($geo['country_code']) ? $geo['country_code'] : '';
	$country = isset($geo['country_name']) ? $geo['country_name'] : '';
	$region  = isset($geo['region_name'])  ? $geo['region_name']  : '';
	$city    = isset($geo['city_name'])    ? $geo['city_name']    : '';
	$zip     = isset($geo['zip_code'])     ? $geo['zip_code']     : '';
	
	$results = array($code, $country, $region, $city, $zip);
	
	return apply_filters('blackhole_geo_data', $results, $geo);
	
}