<?php // Blackhole Pro - Helper functions

if (!defined('ABSPATH')) exit;

// add_filter('nonce_life', 'blackhole_nonce_life');
function blackhole_nonce_life($life) {
	
	return 60; // enabled only when testing
	
}

function blackhole_get_vars() {
	
	$date = blackhole_get_date();
	
	$ip_address = blackhole_get_ip();
	
	$hit_data = blackhole_get_hits($ip_address);
	
	$remote_host = blackhole_get_host($ip_address);
	
	$request_uri = isset($_SERVER['REQUEST_URI'])     ? sanitize_text_field($_SERVER['REQUEST_URI'])     : '';
	$user_agent  = isset($_SERVER['HTTP_USER_AGENT']) ? sanitize_text_field($_SERVER['HTTP_USER_AGENT']) : '';
	$referrer    = isset($_SERVER['HTTP_REFERER'])    ? sanitize_text_field($_SERVER['HTTP_REFERER'])    : '';
	$protocol    = isset($_SERVER['SERVER_PROTOCOL']) ? sanitize_text_field($_SERVER['SERVER_PROTOCOL']) : '';
	$method      = isset($_SERVER['REQUEST_METHOD'])  ? sanitize_text_field($_SERVER['REQUEST_METHOD'])  : '';
	
	$hits   = isset($hit_data['hits'])   ? $hit_data['hits']   : 1;
	$latest = isset($hit_data['latest']) ? $hit_data['latest'] : $date;
	
	$vars = array(
			'ip_address'  => $ip_address, 
			'request_uri' => $request_uri, 
			'remote_host' => $remote_host, 
			'user_agent'  => $user_agent, 
			'referrer'    => $referrer, 
			'protocol'    => $protocol, 
			'method'      => $method, 
			'date'        => $date, 
			'hits'        => $hits, 
			'latest'      => $latest,
		);
	
	return apply_filters('blackhole_vars', $vars);
	
}

function blackhole_get_host($ip_address) {
	
	$remote_host = '';
	
	if (filter_var($ip_address, FILTER_VALIDATE_IP) && apply_filters('blackhole_enable_host_check', true)) {
		
		$remote_host = isset($ip_address) ? sanitize_text_field(gethostbyaddr($ip_address)) : '';
		
		if (strstr($remote_host, ', ')) {
			
			$ips = explode(', ', $remote_host);
			
			$remote_host = $ips[0];
			
		}
		
	}
	
	return $remote_host;
	
}

function blackhole_get_deps() {
	
	global $blackhole_options;
	
	$enabled = isset($blackhole_options['warning_message']) ? esc_attr($blackhole_options['warning_message']) : 'default';
	
	$template = BBB_DIR .'/templates/warning-'. $enabled .'.php';
	
	$filename = apply_filters('blackhole_template_filename', 'warning-custom');
	
	$custom = get_stylesheet_directory() .'/'. $filename .'.php';
	
	$custom_dir = apply_filters('blackhole_custom_dir', 'blackhole');
	
	$custom_alt = WP_CONTENT_DIR .'/'. $custom_dir .'/'. $filename .'.php';
	
	if ($enabled === 'custom') {
		
		if (file_exists($custom_alt)) $template = $custom_alt;
		
		elseif (file_exists($custom)) $template = $custom;
		
	}
	
	require_once 'blackhole-lookup.php';
	require_once $template;
	
}

function blackhole_abort($vars) {
	
	global $blackhole_options;
	
	$ignore_loggedin = isset($blackhole_options['disable_login']) ? $blackhole_options['disable_login'] : true;
	$ignore_backend  = apply_filters('blackhole_ignore_backend', true);
	$ignore_login    = apply_filters('blackhole_ignore_login',   true);
	
	if (
		
		(wp_doing_ajax()) || 
		(wp_doing_cron()) || 
		(blackhole_is_tty()) || 
		(is_admin() && $ignore_backend) || 
		(is_user_logged_in() && $ignore_loggedin) || 
		(blackhole_is_login($vars) && $ignore_login) || 
		(blackhole_whitelist($vars))
		
	) return true;
	
	return false;
	
}

function blackhole_blacklist($vars) {
	
	global $blackhole_options;
	
	$user_agent = isset($vars['user_agent']) ? $vars['user_agent'] : false;
	
	$blacklist_bots = isset($blackhole_options['bot_blacklist']) ? $blackhole_options['bot_blacklist'] : '';
	$blacklist_bots = array_filter(array_map('trim', explode(',', $blacklist_bots)));
	
	if (!empty($blacklist_bots)) {
		
		foreach ($blacklist_bots as $bot) {
			
			if (stripos($user_agent, $bot) !== false) {
				
				return true;
				
			}
			
		}
		
	}
	
	return false;
	
}

function blackhole_whitelist($vars) {
	
	global $blackhole_options;
	
	$user_agent = isset($vars['user_agent']) ? $vars['user_agent'] : false;
	$ip_address = isset($vars['ip_address']) ? $vars['ip_address'] : false;
	
	// bots
	
	$whitelist_bots = isset($blackhole_options['bot_whitelist']) ? $blackhole_options['bot_whitelist'] : '';
	$whitelist_bots = array_filter(array_map('trim', explode(',', $whitelist_bots)));
	
	if (!empty($whitelist_bots)) {
		
		foreach ($whitelist_bots as $bot) {
			
			if (stripos($user_agent, $bot) !== false) {
				
				return true;
				
			}
			
		}
		
	}
	
	// ips
	
	$whitelist_ips = isset($blackhole_options['ip_whitelist']) ? $blackhole_options['ip_whitelist'] : '';
	$whitelist_ips = array_filter(array_map('trim', explode("\n", $whitelist_ips)));
	
	foreach ($whitelist_ips as $ip) {
		
		if (strpos($ip, '/') === false) {
			
			if (substr($ip_address, 0, strlen($ip)) === $ip) {
				
				return true;
				
			}
			
		} else {
			
			if (blackhole_ip_in_range($ip_address, $ip)) {
				
				return true;
				
			}
			
		}
		
	}
	
	return false;
	
}

function blackhole_get_bot_id() {
	
	global $blackhole_badbots;
	
	$array = array();
	
	foreach ($blackhole_badbots as $badbot) {
		
		foreach ($badbot as $key => $value) {
			
			if ($key === 'bot_id') $array[] = $value;
			
		}
		
	}
	
	if (empty($array)) $array = array(0);
	
	$bot_id = max($array) + 1;
	
	return $bot_id;
	
}

function blackhole_get_hits($ip_address) {
	
	global $blackhole_badbots;
	
	$hits = array('hits' => 1, 'latest' => blackhole_get_date());
	
	if (is_array($blackhole_badbots)) {
		
		foreach ($blackhole_badbots as $badbot) {
			
			if (isset($badbot['ip_address']) && $badbot['ip_address'] === $ip_address) {
				
				if (isset($badbot['hits']))   $hits['hits']   = $badbot['hits'];
				if (isset($badbot['latest'])) $hits['latest'] = $badbot['latest'];
				
			}
			
		}
		
	}
	
	return $hits;
	
}

function blackhole_redirect_whitelist($vars) {
	
	global $blackhole_options;
	
	$redirect = isset($blackhole_options['whitelist_redirect']) ? esc_url_raw($blackhole_options['whitelist_redirect']) : false;
	
	if ($redirect && blackhole_whitelist($vars)) {
		
		wp_redirect($redirect);
		
		exit();
		
	}
	
}

function blackhole_redirect_blocked() {
	
	global $blackhole_options;
	
	$redirect = isset($blackhole_options['blocked_redirect']) ? esc_url_raw($blackhole_options['blocked_redirect']) : false;
	
	if ($redirect) {
		
		wp_redirect($redirect);
		
		exit();
		
	}
	
}

function blackhole_disable_cache() {
	
	if (!defined('DONOTCACHEPAGE')) define('DONOTCACHEPAGE', true);
	
	if (
		isset($GLOBALS['wp_fastest_cache']) && 
		is_object($GLOBALS['wp_fastest_cache']) && 
		method_exists($GLOBALS['wp_fastest_cache'], 'deleteCache') && 
		is_callable(array($GLOBALS['wp_fastest_cache'], 'deleteCache'))
	) {
		
		$GLOBALS['wp_fastest_cache']->deleteCache();
		
	}
	
	return DONOTCACHEPAGE;
	
}

function blackhole_clear_cache() {
	
	if (function_exists('w3tc_pgcache_flush')) w3tc_pgcache_flush();
	
	if (function_exists('wp_cache_clear_cache')) wp_cache_clear_cache();
	
	if (isset($GLOBALS['wp_fastest_cache']) && method_exists($GLOBALS['wp_fastest_cache'], 'deleteCache')) $GLOBALS['wp_fastest_cache']->deleteCache();
	
}

function blackhole_is_tty() {
	
	if (function_exists('posix_isatty')) {
		
		if (defined('STDOUT')) {
			
			if (posix_isatty(STDOUT)) return true;
			
		}
		
	}
	
	return false;
	
}

function blackhole_is_login($vars) {
	
	$is_login = false;
	
	$url = isset($vars['request_uri']) ? $vars['request_uri'] : null;
	
	if (stripos($url, 'wp-login.php') !== false) $is_login = true;
	
	if (isset($GLOBALS['pagenow']) && $GLOBALS['pagenow'] === 'wp-login.php') $is_login = true;
	
	return $is_login;
	
}

function blackhole_get_date() {
	
	$date_format = 'Y/m/d';
	
	$time_format = 'h:i:s a';
	
	if (function_exists('current_datetime')) {
		
		$format = $date_format .' \@ '. $time_format;
		
		$date = current_datetime()->format($format);
		
	} else {
		
		$date = date_i18n($date_format, current_time('timestamp')) .' \@ '. date_i18n($time_format, current_time('timestamp'));
		
	}
	
	return apply_filters('blackhole_date', $date);
	
}

function blackhole_truncate($string, $length = 100, $append = '...') {
	
	$string = trim($string);
	
	if (strlen($string) > $length) {
		
		$string = wordwrap($string, $length);
		$string = explode("\n", $string, 2);
		$string = $string[0] . $append;
		
	}
	
	return $string;
	
}
