<?php // Blackhole Pro - Trigger functionality

if (!defined('ABSPATH')) exit;

function blackhole_trigger_nonce() {
	
	return wp_create_nonce('blackhole_trigger');
	
}

function blackhole_trigger_hook() {
	
	global $blackhole_options;
	
	$trigger_location = isset($blackhole_options['trigger_location']) ? $blackhole_options['trigger_location'] : 'wp_footer';
	
	if (!function_exists('wp_body_open')) $trigger_location = 'wp_footer';
	
	$trigger_location = apply_filters('blackhole_trigger_location', $trigger_location);
	
	return $trigger_location;
	
}

function blackhole_trigger_shortcode($auto = false) {
	
	global $blackhole_options;
	
	if (blackhole_trigger_exclude()) return;
	
	$disable = isset($blackhole_options['disable_trigger']) ? $blackhole_options['disable_trigger'] : false;
	
	if ($disable && $auto) return;
	
	$nonce = blackhole_trigger_nonce();
	
	$keyword = isset($blackhole_options['trigger_keyword']) ? $blackhole_options['trigger_keyword'] : 'blackhole';
	
	$href  = site_url('/?'. $keyword .'='. $nonce);
	
	$text  = esc_html__(get_bloginfo('name'));
	
	$title = esc_html__('Do NOT follow this link or you will be banned from the site!', 'blackhole-pro');
	
	$image = '<img src="'. BBB_URL .'img/1x1.gif" alt="" />';
	
	$format = isset($blackhole_options['trigger_format']) ? $blackhole_options['trigger_format'] : 'default';
	
	if ($format === 'custom') {
		
		$link = blackhole_trigger_custom($href, $text, $image, $nonce, $title);
		
	} elseif ($format === 'image') {
		
		$link = blackhole_trigger_image($href, $text, $image, $title);
		
	} else {
		
		$link = blackhole_trigger_default($href, $text, $title);
		
	}
	
	do_action('blackhole_trigger', $href, $text, $image, $nonce, $link, $title);
	
	return $link;
	
}

function blackhole_trigger() {
	
	echo blackhole_trigger_shortcode(true);
	
}

function blackhole_trigger_default($href, $text, $title) {
	
	$link = '<a rel="nofollow" style="display:none" href="'. $href .'" title="'. $title .'">'. $text .'</a>'. "\n";
	
	return apply_filters('blackhole_trigger_default', $link, $href, $text, $title);
	
}

function blackhole_trigger_image($href, $text, $image, $title) {
	
	$link = '<a rel="nofollow" style="display:none" href="'. $href .'" title="'. $title .'">'. $image .'</a>'. "\n";
	
	return apply_filters('blackhole_trigger_image', $link, $href, $text, $image, $title);
	
}

function blackhole_trigger_custom($href, $text, $image, $nonce, $title) {
	
	global $blackhole_options;
	
	$default = blackhole_trigger_default($href, $text, $title);
	
	$custom = isset($blackhole_options['trigger_custom']) ? $blackhole_options['trigger_custom'] : $default;
	
	$patterns = array();
	$patterns[0] = "/\[href\]/";
	$patterns[1] = "/\[text\]/";
	$patterns[2] = "/\[image\]/";
	$patterns[3] = "/\[nonce\]/";
	$patterns[4] = "/\[title\]/";
	
	$replacements = array();
	$replacements[0] = $href;
	$replacements[1] = $text;
	$replacements[2] = $image;
	$replacements[3] = $nonce;
	$replacements[4] = $title;
	
	$link = preg_replace($patterns, $replacements, $custom) . "\n";
	
	return apply_filters('blackhole_trigger_custom', $link);
	
}

function blackhole_trigger_exclude() {
	
	global $blackhole_options;
	
	if (!is_singular()) return false;
	
	$current = get_the_ID();
	
	$ids = isset($blackhole_options['excluded_pages']) ? $blackhole_options['excluded_pages'] : '';
	
	$ids = array_map('trim', explode(',', $ids));
	
	foreach ($ids as $id) {
		
		if (intval($id) === $current) return true;
		
	}
	
	return false;
	
}