<?php // Blackhole Pro - Register Settings

if (!defined('ABSPATH')) exit;

function blackhole_register_settings() {
	
	// register_setting( $option_group, $option_name, $sanitize_callback );
	register_setting('blackhole_pro_options', 'blackhole_pro_options', 'blackhole_validate_options');
	
	// add_settings_section( $id, $title, $callback, $page ); 
	add_settings_section('general', esc_html__('Set the Controls..', 'blackhole-pro'), 'blackhole_settings_section_general', 'blackhole_pro_options');
	
	// add_settings_field( $id, $title, $callback, $page, $section, $args );
	add_settings_field('robots_rules',  esc_html__('Robots Rules',    'blackhole-pro'), 'blackhole_callback_robots',   'blackhole_pro_options', 'general', array('id' => 'robots_rules',  'label' => esc_html__('Add these rules to your site&rsquo;s robots.txt file', 'blackhole-pro')));
	add_settings_field('disable_login', esc_html__('Logged-in Users', 'blackhole-pro'), 'blackhole_callback_checkbox', 'blackhole_pro_options', 'general', array('id' => 'disable_login', 'label' => esc_html__('Disable Blackhole for logged-in users', 'blackhole-pro')));
	add_settings_field('hits_required', esc_html__('Threshold',       'blackhole-pro'), 'blackhole_callback_number',   'blackhole_pro_options', 'general', array('id' => 'hits_required', 'label' => esc_html__('Number of hits before blocking', 'blackhole-pro'), 'min' => 1, 'max' => 999));
	
	add_settings_section('alerts', esc_html__('Email Alerts', 'blackhole-pro'), 'blackhole_settings_section_alerts', 'blackhole_pro_options');
	
	add_settings_field('email_alerts',    esc_html__('Email Alerts',    'blackhole-pro'), 'blackhole_callback_checkbox', 'blackhole_pro_options', 'alerts', array('id' => 'email_alerts',  'label' => esc_html__('Enable email alerts', 'blackhole-pro')));
	add_settings_field('email_address',   esc_html__('Email Address',   'blackhole-pro'), 'blackhole_callback_text',     'blackhole_pro_options', 'alerts', array('id' => 'email_address', 'label' => esc_html__('Email address for alerts', 'blackhole-pro')));
	add_settings_field('email_from',      esc_html__('Email From',      'blackhole-pro'), 'blackhole_callback_text',     'blackhole_pro_options', 'alerts', array('id' => 'email_from',    'label' => esc_html__('Email address for &ldquo;From&rdquo; header', 'blackhole-pro')));
	add_settings_field('alert_type',      esc_html__('Alert Type',      'blackhole-pro'), 'blackhole_callback_select',   'blackhole_pro_options', 'alerts', array('id' => 'alert_type',    'label' => esc_html__('Type of email alert', 'blackhole-pro')));
	add_settings_field('alert_message',   esc_html__('Custom Message',  'blackhole-pro'), 'blackhole_callback_textarea', 'blackhole_pro_options', 'alerts', array('id' => 'alert_message', 'label' => esc_html__('Custom alert message', 'blackhole-pro') .' <span class="bbb-note">'. esc_html__('(when Custom is selected in previous setting, see Help tab for shortcodes &amp; more info)', 'blackhole-pro') .'</span>'));
	add_settings_field('reset_reports',   esc_html__('Reset Reports',   'blackhole-pro'), 'blackhole_callback_reports',  'blackhole_pro_options', 'alerts', array('id' => 'reset_reports', 'label' => esc_html__('Reset reports for email alerts', 'blackhole-pro')));
	
	add_settings_section('frontend', esc_html__('Front-End Display', 'blackhole-pro'), 'blackhole_settings_section_frontend', 'blackhole_pro_options');
	
	add_settings_field('warning_message',  esc_html__('Warning Message', 'blackhole-pro'), 'blackhole_callback_select',   'blackhole_pro_options', 'frontend', array('id' => 'warning_message',  'label' => esc_html__('Warning message for bad bots', 'blackhole-pro')));
	add_settings_field('blocked_message',  esc_html__('Blocked Message', 'blackhole-pro'), 'blackhole_callback_select',   'blackhole_pro_options', 'frontend', array('id' => 'blocked_message',  'label' => esc_html__('Message displayed to blocked bots', 'blackhole-pro')));
	add_settings_field('blocked_custom',   esc_html__('Custom Message',  'blackhole-pro'), 'blackhole_callback_textarea', 'blackhole_pro_options', 'frontend', array('id' => 'blocked_custom',   'label' => esc_html__('Custom blocked message', 'blackhole-pro') .' <span class="bbb-note">'. esc_html__('(when Custom is selected in previous setting)', 'blackhole-pro') .'</span>'));
	add_settings_field('blocked_redirect', esc_html__('Custom Redirect', 'blackhole-pro'), 'blackhole_callback_text',     'blackhole_pro_options', 'frontend', array('id' => 'blocked_redirect', 'label' => esc_html__('Enter a custom URL for blocked bots', 'blackhole-pro') .' <span class="bbb-note">'. esc_html__('(leave blank to use default)', 'blackhole-pro') .'</span>'));
	
	add_settings_section('trigger', esc_html__('Blackhole Trigger', 'blackhole-pro'), 'blackhole_settings_section_trigger', 'blackhole_pro_options');
	
	add_settings_field('trigger_format',   esc_html__('Trigger Format',   'blackhole-pro'), 'blackhole_callback_select',   'blackhole_pro_options', 'trigger', array('id' => 'trigger_format',   'label' => esc_html__('Format of the blackhole trigger link',     'blackhole-pro')));
	add_settings_field('trigger_location', esc_html__('Trigger Location', 'blackhole-pro'), 'blackhole_callback_select',   'blackhole_pro_options', 'trigger', array('id' => 'trigger_location', 'label' => esc_html__('Location of the blackhole trigger link',   'blackhole-pro')));
	add_settings_field('trigger_custom',   esc_html__('Custom Trigger',   'blackhole-pro'), 'blackhole_callback_textarea', 'blackhole_pro_options', 'trigger', array('id' => 'trigger_custom',   'label' => esc_html__('Custom trigger link',                      'blackhole-pro') .' <span class="bbb-note">'. esc_html__('(when Custom is selected in previous setting)', 'blackhole-pro') .'</span>'));
	add_settings_field('trigger_keyword',  esc_html__('Custom Keyword',   'blackhole-pro'), 'blackhole_callback_text',     'blackhole_pro_options', 'trigger', array('id' => 'trigger_keyword',  'label' => esc_html__('Custom URL parameter for trigger link',    'blackhole-pro') .' <span class="bbb-note">'. esc_html__('(choose a *unique* alphanumeric string)', 'blackhole-pro') .'</span>'));
	add_settings_field('excluded_pages',   esc_html__('Excluded Pages',   'blackhole-pro'), 'blackhole_callback_text',     'blackhole_pro_options', 'trigger', array('id' => 'excluded_pages',   'label' => esc_html__('Exclude blackhole trigger on these pages', 'blackhole-pro') .' <span class="bbb-note">'. esc_html__('(post/page IDs separated by comma)', 'blackhole-pro') .'</span>'));
	add_settings_field('disable_trigger',  esc_html__('Disable Trigger',  'blackhole-pro'), 'blackhole_callback_checkbox', 'blackhole_pro_options', 'trigger', array('id' => 'disable_trigger',  'label' => esc_html__('Disable automatic trigger link',           'blackhole-pro') .' <span class="bbb-note">'. esc_html__('(advanced use, see Help tab for details)', 'blackhole-pro') .'</span>'));
	
	add_settings_section('botlog', esc_html__('Bad Bots Log', 'blackhole-pro'), 'blackhole_settings_section_botlog', 'blackhole_pro_options');
	
	add_settings_field('botlog_items', esc_html__('Number of Bots', 'blackhole-pro'), 'blackhole_callback_number',        'blackhole_pro_options', 'botlog', array('id' => 'botlog_items', 'label' => esc_html__('Number of bots displayed per page', 'blackhole-pro'), 'min' => 1, 'max' => 999));
	add_settings_field('botlog_reset', esc_html__('Reset Bot Log',  'blackhole-pro'), 'blackhole_callback_reset_badbots', 'blackhole_pro_options', 'botlog', array('id' => 'botlog_reset', 'label' => esc_html__('Clear all bad bots and reset log', 'blackhole-pro')));
	
	add_settings_section('advanced', esc_html__('Advanced Settings', 'blackhole-pro'), 'blackhole_settings_section_advanced', 'blackhole_pro_options');
	
	add_settings_field('bot_blacklist',      esc_html__('Blacklist Bots',     'blackhole-pro'), 'blackhole_callback_textarea', 'blackhole_pro_options', 'advanced', array('id' => 'bot_blacklist',      'label' => esc_html__('User agents that always should be blocked', 'blackhole-pro') .' <span class="bbb-note">'. esc_html__('(separate with commas)', 'blackhole-pro') .'</span>'));
	add_settings_field('bot_whitelist',      esc_html__('Whitelist Bots',     'blackhole-pro'), 'blackhole_callback_textarea', 'blackhole_pro_options', 'advanced', array('id' => 'bot_whitelist',      'label' => esc_html__('User agents that never should be blocked', 'blackhole-pro')  .' <span class="bbb-note">'. esc_html__('(separate with commas)', 'blackhole-pro') .'</span>'));
	add_settings_field('ip_whitelist',       esc_html__('Whitelist IPs',      'blackhole-pro'), 'blackhole_callback_textarea', 'blackhole_pro_options', 'advanced', array('id' => 'ip_whitelist',       'label' => esc_html__('IP addresses that never should be blocked', 'blackhole-pro') .' <span class="bbb-note">'. esc_html__('(one IP address per line)', 'blackhole-pro') .'</span>'));
	add_settings_field('whitelist_redirect', esc_html__('Whitelist Redirect', 'blackhole-pro'), 'blackhole_callback_text',     'blackhole_pro_options', 'advanced', array('id' => 'whitelist_redirect', 'label' => esc_html__('Enter a custom URL for whitelisted bots', 'blackhole-pro') .' <span class="bbb-note">'. esc_html__('(Important: view Help tab for details)', 'blackhole-pro') .'</span>'));
	add_settings_field('status_code',        esc_html__('Status Code',        'blackhole-pro'), 'blackhole_callback_select',   'blackhole_pro_options', 'advanced', array('id' => 'status_code',        'label' => esc_html__('Status code for all blocked bots', 'blackhole-pro')));
	add_settings_field('reset_options',      esc_html__('Reset Options',      'blackhole-pro'), 'blackhole_callback_reset',    'blackhole_pro_options', 'advanced', array('id' => 'reset_options',      'label' => esc_html__('Restore default plugin options', 'blackhole-pro')));
	add_settings_field('rate_plugin',        esc_html__('Rate Plugin',        'blackhole-pro'), 'blackhole_callback_rate',     'blackhole_pro_options', 'advanced', array('id' => 'rate_plugin',        'label' => esc_html__('Show support with a 5-star rating &raquo;', 'blackhole-pro')));
	
}

function blackhole_validate_options($input) {
	
	$allowed_tags     = blackhole_allowed_tags();
	$warning_message  = blackhole_warning_message();
	$blocked_message  = blackhole_blocked_message();
	$trigger_format   = blackhole_trigger_format();
	$trigger_location = blackhole_trigger_location();
	$status_codes     = blackhole_status_codes();
	$alert_type       = blackhole_alert_type();
	
	if (!isset($input['disable_login'])) $input['disable_login'] = null;
	$input['disable_login'] = ($input['disable_login'] == 1 ? 1 : 0);
	
	if (isset($input['hits_required']) && is_numeric($input['hits_required'])) $input['hits_required'] = intval($input['hits_required']);
	
	if (!isset($input['email_alerts'])) $input['email_alerts'] = null;
	$input['email_alerts'] = ($input['email_alerts'] == 1 ? 1 : 0);
	
	if (isset($input['email_address'])) $input['email_address'] = wp_filter_nohtml_kses($input['email_address']);
	
	if (isset($input['email_from'])) $input['email_from'] = wp_filter_nohtml_kses($input['email_from']);
	
	//
	
	if (!isset($input['alert_type'])) $input['alert_type'] = null;
	if (!array_key_exists($input['alert_type'], $alert_type)) $input['alert_type'] = null;
	
	if (isset($input['alert_type'])) {
		
		$type = $input['alert_type'];
		
		if ($type === 'daily' || $type === 'weekly') {
			
			if ($type !== wp_get_schedule('blackhole_send_email_cron')) {
				
				blackhole_cron_deactivation();
				
				wp_schedule_event(time(), $type, 'blackhole_send_email_cron');
				
			}
			
		} else {
			
			blackhole_cron_deactivation();
			
		}
		
	}
	
	//
	
	if (isset($input['alert_message'])) $input['alert_message'] = wp_kses(stripslashes_deep($input['alert_message']), $allowed_tags);
	
	if (!isset($input['warning_message'])) $input['warning_message'] = null;
	if (!array_key_exists($input['warning_message'], $warning_message)) $input['warning_message'] = null;
	
	if (!isset($input['blocked_message'])) $input['blocked_message'] = null;
	if (!array_key_exists($input['blocked_message'], $blocked_message)) $input['blocked_message'] = null;
	
	if (isset($input['blocked_custom'])) $input['blocked_custom'] = wp_kses(stripslashes_deep($input['blocked_custom']), $allowed_tags);
	
	if (isset($input['blocked_redirect'])) $input['blocked_redirect'] = wp_filter_nohtml_kses($input['blocked_redirect']);
	
	if (!isset($input['trigger_format'])) $input['trigger_format'] = null;
	if (!array_key_exists($input['trigger_format'], $trigger_format)) $input['trigger_format'] = null;
	
	if (!isset($input['trigger_location'])) $input['trigger_location'] = null;
	if (!array_key_exists($input['trigger_location'], $trigger_location)) $input['trigger_location'] = null;
	
	if (isset($input['trigger_custom'])) $input['trigger_custom'] = wp_kses(stripslashes_deep($input['trigger_custom']), $allowed_tags);
	
	if (isset($input['trigger_keyword'])) $input['trigger_keyword'] = wp_filter_nohtml_kses($input['trigger_keyword']);
	
	if (isset($input['excluded_pages'])) $input['excluded_pages'] = wp_filter_nohtml_kses($input['excluded_pages']);
	
	if (!isset($input['disable_trigger'])) $input['disable_trigger'] = null;
	$input['disable_trigger'] = ($input['disable_trigger'] == 1 ? 1 : 0);
	
	if (isset($input['botlog_items']) && is_numeric($input['botlog_items'])) $input['botlog_items'] = intval($input['botlog_items']);
	
	if (isset($input['bot_blacklist'])) $input['bot_blacklist'] = wp_filter_nohtml_kses($input['bot_blacklist']);
	
	if (isset($input['bot_whitelist'])) $input['bot_whitelist'] = wp_filter_nohtml_kses($input['bot_whitelist']);
	
	if (isset($input['ip_whitelist'])) $input['ip_whitelist'] = wp_filter_nohtml_kses($input['ip_whitelist']);
	
	if (isset($input['whitelist_redirect'])) $input['whitelist_redirect'] = wp_filter_nohtml_kses($input['whitelist_redirect']);
	
	if (!isset($input['status_code'])) $input['status_code'] = null;
	if (!array_key_exists($input['status_code'], $status_codes)) $input['status_code'] = null;
	
	return $input;
	
}

function blackhole_allowed_tags() {
	
	$allowed_tags = wp_kses_allowed_html('post');
	
	$allowed_tags['a'] = array_merge($allowed_tags['a'], array('style' => true));
	$allowed_tags['style'] = array('media' => true, 'scoped' => true, 'type' => true);
	$allowed_tags['script'] = array('async' => true, 'charset' => true, 'defer' => true, 'src' => true, 'type' => true);
	
	return apply_filters('blackhole_allowed_tags', $allowed_tags);
	
}

function blackhole_settings_section_general() {
	
	echo '<p>'. esc_html__('Thanks for using Blackhole Pro. May your site be free of bad bots. Visit the', 'blackhole-pro');
	
	echo ' <strong>'. esc_html__('Help tab', 'blackhole-pro') .'</strong> '. esc_html__('for complete documentation and tips.', 'blackhole-pro') .'</p>';
	
	echo '<p><strong>'. esc_html__('Important', 'blackhole-pro') .'</strong> '. esc_html__('note about', 'blackhole-pro');
	
	echo ' <a target="_blank" rel="noopener noreferrer" href="https://plugin-planet.com/blackhole-pro-cache-plugins/">'. esc_html__('cache plugins', 'blackhole-pro') .'&nbsp;&raquo;</a></p>';
	
}

function blackhole_settings_section_frontend() {
	
	echo '<p>'. esc_html__('Customize the appearance of the Blackhole on the frontend.', 'blackhole-pro') .'</p>';
	
}

function blackhole_settings_section_trigger() {
	
	echo '<p>'. esc_html__('Customize the trigger link with these settings.', 'blackhole-pro') .'</p>';
	
}

function blackhole_settings_section_botlog() {
	
	echo '<p>'. esc_html__('These settings control the', 'blackhole-pro');
	echo ' <a href="'. admin_url('admin.php?page=blackhole_badbots') .'">'. esc_html__('Bad Bots Log', 'blackhole-pro') .'</a>.</p>';
	
}

function blackhole_settings_section_advanced() {
	
	echo '<p>'. esc_html__('Advanced settings. Read the Help tab for more information.', 'blackhole-pro') .'</p>';
	
}

function blackhole_settings_section_alerts() {
	
	echo '<p>'. esc_html__('Stay aware with Blackhole Email Alerts.', 'blackhole-pro') .'</p>';
	
}

function blackhole_blocked_message() {
	
	$blocked_message = array(
		'default' => array(
			'value' => 'default',
			'label' => esc_html__('Default Message', 'blackhole-pro'),
		),
		'custom' => array(
			'value' => 'custom',
			'label' => esc_html__('Custom Message', 'blackhole-pro'),
		),
		'void' => array(
			'value' => 'void',
			'label' => esc_html__('Into the Void', 'blackhole-pro'),
		),
		'redgiant' => array(
			'value' => 'redgiant',
			'label' => esc_html__('Red Giant', 'blackhole-pro'),
		),
	);
	return $blocked_message;
	
}

function blackhole_warning_message() {
	
	$warning_message = array(
		'default' => array(
			'value' => 'default',
			'label' => esc_html__('Default Message', 'blackhole-pro'),
		),
		'custom' => array(
			'value' => 'custom',
			'label' => esc_html__('Custom Message', 'blackhole-pro'),
		),
		'simple' => array(
			'value' => 'simple',
			'label' => esc_html__('Simple Message', 'blackhole-pro'),
		),
		'basic' => array(
			'value' => 'basic',
			'label' => esc_html__('Basic Message', 'blackhole-pro'),
		),
	);
	return $warning_message;
	
}

function blackhole_trigger_format() {
	
	$trigger_format = array(
		'default' => array(
			'value' => 'default',
			'label' => esc_html__('Text Link (Default)', 'blackhole-pro'),
		),
		'image' => array(
			'value' => 'image',
			'label' => esc_html__('Image Link', 'blackhole-pro'),
		),
		'custom' => array(
			'value' => 'custom',
			'label' => esc_html__('Custom Link', 'blackhole-pro'),
		),
	);
	return $trigger_format;
	
}

function blackhole_trigger_location() {
	
	$trigger_location = array(
		'wp_footer' => array(
			'value' => 'wp_footer',
			'label' => esc_html__('Footer (Default)', 'blackhole-pro'),
		),
		'wp_body_open' => array(
			'value' => 'wp_body_open',
			'label' => esc_html__('Body open tag', 'blackhole-pro'),
		),
	);
	return $trigger_location;
	
}

function blackhole_alert_type() {
	
	$alert_type = array(
		'default' => array(
			'value' => 'default',
			'label' => esc_html__('Default Message', 'blackhole-pro'),
		),
		'simple' => array(
			'value' => 'simple',
			'label' => esc_html__('Simple Message', 'blackhole-pro'),
		),
		'custom' => array(
			'value' => 'custom',
			'label' => esc_html__('Custom Message', 'blackhole-pro'),
		),
		'daily' => array(
			'value' => 'daily',
			'label' => esc_html__('Daily Report', 'blackhole-pro'),
		),
		'weekly' => array(
			'value' => 'weekly',
			'label' => esc_html__('Weekly Report', 'blackhole-pro'),
		),
	);
	return $alert_type;
	
}

function blackhole_callback_text($args) {
	
	global $blackhole_options;
	
	$id = isset($args['id']) ? $args['id'] : '';
	$label = isset($args['label']) ? $args['label'] : '';
	
	$value = isset($blackhole_options[$id]) ? sanitize_text_field($blackhole_options[$id]) : '';
	
	echo '<input name="blackhole_pro_options['. $id .']" type="text" size="40" value="'. $value .'" />';
	echo '<label class="bbb-label" for="blackhole_pro_options['. $id .']">'. $label .'</label>';
	
}

function blackhole_callback_number($args) {
	
	global $blackhole_options;
	
	$id = isset($args['id']) ? $args['id'] : '';
	$label = isset($args['label']) ? $args['label'] : '';
	$min = isset($args['min']) ? $args['min'] : 0;
	$max = isset($args['max']) ? $args['max'] : null;
	$value = isset($blackhole_options[$id]) ? sanitize_text_field($blackhole_options[$id]) : '';
	
	echo '<input name="blackhole_pro_options['. $id .']" type="number" class="small-text" min="'. $min .'" max="'. $max .'" value="'. $value .'" /> ';
	echo '<label class="bbb-label bbb-inline-block" for="blackhole_pro_options['. $id .']">'. $label .'</label>';
	
}

function blackhole_callback_textarea($args) {
	
	global $blackhole_options;
	
	$allowed_tags = blackhole_allowed_tags();
	
	$id = isset($args['id']) ? $args['id'] : '';
	$label = isset($args['label']) ? $args['label'] : '';
	$value = isset($blackhole_options[$id]) ? wp_kses(stripslashes_deep($blackhole_options[$id]), $allowed_tags) : '';
	
	$class = ($id === 'alert_message' || $id === 'blocked_custom' || $id === 'trigger_custom') ? ' class="code"' : '';
	
	$message = '<label class="bbb-label textarea-message"><span class="message-dot"></span>'. esc_html__('Read important message about this setting in the Help tab above', 'blackhole-pro') .' <span class="bbb-light-text">('. esc_html__('under &ldquo;Advanced Settings&rdquo;', 'blackhole-pro') .')</span></label>';
	
	if ($id === 'bot_whitelist') {
		
		echo $message;
		
	} elseif ($id === 'ip_whitelist') {
		
		$value = explode(",", $value);
		$value = array_filter(array_map('trim', $value));
		$value = array_unique($value);
		$value = implode("\n", $value);
		
	}
	
	echo '<textarea name="blackhole_pro_options['. $id .']"'. $class .' rows="3" cols="50">'. $value .'</textarea>';
	echo '<label class="bbb-label" for="blackhole_pro_options['. $id .']">'. $label .'</label>';
	
}

function blackhole_callback_checkbox($args) {
	
	global $blackhole_options;
	
	$id = isset($args['id']) ? $args['id'] : '';
	$label = isset($args['label']) ? $args['label'] : '';
	$checked = isset($blackhole_options[$id]) ? checked($blackhole_options[$id], 1, false) : '';
	
	echo '<input name="blackhole_pro_options['. $id .']" type="checkbox" value="1" '. $checked .' /> ';
	echo '<label class="bbb-label bbb-inline-block" for="blackhole_pro_options['. $id .']">'. $label .'</label>';
	
}

function blackhole_callback_radio($args) {
	
	global $blackhole_options;
	
	$options_array = array();
	if     ($args['id'] === 'blocked_message') $options_array = blackhole_blocked_message();
	elseif ($args['id'] === 'warning_message') $options_array = blackhole_warning_message();
	
	$id = isset($args['id']) ? $args['id'] : '';
	$label = isset($args['label']) ? $args['label'] : '';
	$value = isset($blackhole_options[$id]) ? sanitize_text_field($blackhole_options[$id]) : '';
	
	echo '<label class="bbb-label bbb-inline-block" for="blackhole_pro_options['. $id .']">'. $label .'</label>';
	echo '<ul>';
	
	foreach ($options_array as $option) {
		
		$checked = '';
		if ($value == $option['value']) $checked = ' checked="checked"';
		
		echo '<li><input type="radio" name="blackhole_pro_options['. $id .']" value="'. $option['value'] .'"'. $checked .' /> '. $option['label'] .'</li>';
		
	}
	echo '</ul>';
	
}

function blackhole_callback_select($args) {
	
	global $blackhole_options;
	
	$options_array = array();
	if     ($args['id'] === 'blocked_message')  $options_array = blackhole_blocked_message();
	elseif ($args['id'] === 'warning_message')  $options_array = blackhole_warning_message();
	elseif ($args['id'] === 'trigger_format')   $options_array = blackhole_trigger_format();
	elseif ($args['id'] === 'trigger_location') $options_array = blackhole_trigger_location();
	elseif ($args['id'] === 'status_code')      $options_array = blackhole_status_codes();
	elseif ($args['id'] === 'alert_type')       $options_array = blackhole_alert_type();
	
	$id    = isset($args['id']) ? $args['id'] : '';
	$label = isset($args['label']) ? $args['label'] : '';
	$value = isset($blackhole_options[$id]) ? sanitize_text_field($blackhole_options[$id]) : '';
	
	echo '<select name="blackhole_pro_options['. $id .']">';
	
	foreach ($options_array as $option) {
		echo '<option '. selected($option['value'], $value, false) .' value="'. $option['value'] .'">'. $option['label'] .'</option>';
	}
	echo '</select> <label class="bbb-label bbb-inline-block" for="blackhole_pro_options['. $id .']">'. $label .'</label>';
	
}

function blackhole_callback_reset($args) {
	
	$nonce = wp_create_nonce('blackhole_reset_options');
	$href  = esc_url(add_query_arg(array('reset-options-verify' => $nonce), admin_url('admin.php?page=blackhole_settings')));
	$label = isset($args['label']) ? $args['label'] : esc_html__('Restore default plugin options', 'blackhole-pro');
	
	echo '<a class="bbb-reset-options" href="'. $href .'">'. $label .'</a>';
	
}

function blackhole_callback_reports($args) {
	
	$nonce = wp_create_nonce('blackhole_reset_reports');
	$href  = esc_url(add_query_arg(array('reset-reports-verify' => $nonce), admin_url('admin.php?page=blackhole_settings')));
	$label = isset($args['label']) ? $args['label'] : esc_html__('Reset reports for email alerts', 'blackhole-pro');
	
	echo '<a class="bbb-reset-reports" href="'. $href .'">'. $label .'</a>';
	
}

function blackhole_callback_reset_badbots() {
	
	$nonce = wp_create_nonce('blackhole_reset_badbots');
	$href = esc_url(add_query_arg(array('reset-badbots-verify' => $nonce), admin_url('admin.php?page=blackhole_badbots')));
	$label = isset($args['label']) ? $args['label'] : esc_html__('Clear all bad bots and reset log', 'blackhole-pro');
	
	echo '<a class="bbb-reset-badbots" href="'. $href .'">'. $label .'</a>';
	
}

function blackhole_callback_rate($args) {
	
	$label = isset($args['label']) ? $args['label'] : esc_html__('Show support with a 5-star rating &raquo;', 'blackhole-pro');
	$href  = 'https://wordpress.org/support/plugin/blackhole-bad-bots/reviews/?rate=5#new-post';
	$title = esc_attr__('Help keep Blackhole going strong! A huge THANK YOU for your support!', 'blackhole-pro');
	
	echo '<a target="_blank" rel="noopener noreferrer" class="bbb-rate-plugin" href="'. $href .'" title="'. $title .'">'. $label .'</a>';
	
}

function blackhole_callback_robots() {
	
	$rules = blackhole_robots();
	
	if (empty($rules)) {
		
		$display = '<em class="bbb-warning">'. esc_html__('Please check WP General Settings.', 'blackhole-pro') .'</em>';
		
	} else {
		
		$protocol = is_ssl() ? 'https://' : 'http://';
		
		$url = $protocol . blackhole_domain() . '/robots.txt';
		
		$title = __('View your site&rsquo;s robots.txt', 'blackhole-pro');
		
		$text = __('robots.txt file', 'blackhole-pro');
		
		$link = '<a target="_blank" rel="noopener noreferrer" href="'. esc_url($url) .'" title="'. esc_attr($title) .'">'. esc_html($text) .'</a>';
		
		$display  = '<div><em>'. esc_html__('Add the following rules to your site&rsquo;s ', 'blackhole-pro') . $link .':</em></div>';
		
		$display .= '<pre>'. trim($rules) .'</pre>';
		
		$display .= '<div><em>'. esc_html__('If you are using WP-generated robots.txt, these rules are added automatically.', 'blackhole-pro') .'</em></div>';
		
	}
	
	echo $display;
	
}
