<?php // Blackhole Pro - Reset Settings

if (!defined('ABSPATH')) exit;

function blackhole_admin_notices() {
	
	$screen_id = blackhole_get_current_screen_id();
	
	if (($screen_id === 'toplevel_page_blackhole_settings') || ($screen_id === 'blackhole-pro_page_blackhole_badbots')) {
		
		if (isset($_GET['reset-options'])) {
			
			if ($_GET['reset-options'] === 'true') : ?>
				
				<div class="notice notice-success is-dismissible"><p><strong><?php esc_html_e('Default options restored.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php else : ?>
				
				<div class="notice notice-info is-dismissible"><p><strong><?php esc_html_e('No changes made.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php endif;
			
		} elseif (isset($_GET['reset-reports'])) {
			
			if ($_GET['reset-reports'] === 'true') : ?>
				
				<div class="notice notice-success is-dismissible"><p><strong><?php esc_html_e('Reports reset for all bots.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php else : ?>
				
				<div class="notice notice-info is-dismissible"><p><strong><?php esc_html_e('No changes made.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php endif;
			
		} elseif (isset($_GET['reset-badbots'])) {
			
			if ($_GET['reset-badbots'] === 'true') : ?>
				
				<div class="notice notice-success is-dismissible"><p><strong><?php esc_html_e('Bad bots cleared.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php else : ?>
				
				<div class="notice notice-info is-dismissible"><p><strong><?php esc_html_e('No changes made.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php endif;
			
		} elseif (isset($_GET['refresh-geo']) && isset($_GET['bot-id'])) {
			
			if ($_GET['refresh-geo'] === 'true') : 
				
				$bot_id = esc_html($_GET['bot-id']); ?>
				
				<div class="notice notice-success is-dismissible"><p><strong><?php esc_html_e('GeoIP updated for bot ID:', 'blackhole-pro'); ?> <?php echo $bot_id; ?></strong></p></div>
				
			<?php else : ?>
				
				<div class="notice notice-info is-dismissible"><p><strong><?php esc_html_e('No changes made.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php endif;
			
		} elseif (isset($_GET['update-bot'])) {
			
			if ($_GET['update-bot'] === 'both') : ?>
				
				<div class="notice notice-success is-dismissible"><p><strong><?php esc_html_e('Selected bots added to whitelist and deleted.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php elseif ($_GET['update-bot'] === 'whitelist') : ?>
				
				<div class="notice notice-success is-dismissible"><p><strong><?php esc_html_e('Selected bots added to whitelist.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php elseif ($_GET['update-bot'] === 'delete') : ?>
				
				<div class="notice notice-success is-dismissible"><p><strong><?php esc_html_e('Selected bots deleted.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php else : ?>
				
				<div class="notice notice-info is-dismissible"><p><strong><?php esc_html_e('No changes made.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php endif;
			
		} elseif (isset($_GET['add-badbot'])) {
			
			if ($_GET['add-badbot'] === 'true') : 
			
			$bot_id = (isset($_GET['bot-id'])) ? sanitize_text_field($_GET['bot-id']) : 'error-id'; ?>
				
				<div class="notice notice-success is-dismissible"><p><strong><?php echo esc_html__('Bot added. Bot ID: ', 'blackhole-pro') . $bot_id; ?></strong></p></div>
				
			<?php elseif ($_GET['add-badbot'] === 'exists') : 
				
				$bot_ip = (isset($_GET['bot-ip'])) ? sanitize_text_field($_GET['bot-ip']) : 'error-ip'; ?>
				
				<div class="notice notice-warning is-dismissible"><p><strong><?php echo esc_html__('Bot not added: IP address ', 'blackhole-pro') . $bot_ip . esc_html__(' already exists.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php elseif ($_GET['add-badbot'] === 'ip-required') : ?>
				
				<div class="notice notice-warning is-dismissible"><p><strong><?php esc_html_e('Bot not added: IP address required.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php else : ?>
				
				<div class="notice notice-info is-dismissible"><p><strong><?php esc_html_e('No bots added.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php endif;
			
		}
		
	}
	
	if ($screen_id === 'toplevel_page_blackhole_settings') {
		
		if (!blackhole_check_date_expired() && !blackhole_dismiss_notice_check()) {
			
			?>
			
			<div class="notice notice-success">
				<p>
					<strong><?php esc_html_e('Go Pro!', 'blackhole-pro'); ?></strong> 
					<?php esc_html_e('Save 30% on our', 'blackhole-pro'); ?> 
					<a target="_blank" rel="noopener noreferrer" href="https://plugin-planet.com/"><?php esc_html_e('Pro WordPress plugins', 'blackhole-pro'); ?></a> 
					<?php esc_html_e('and', 'blackhole-pro'); ?> 
					<a target="_blank" rel="noopener noreferrer" href="https://books.perishablepress.com/"><?php esc_html_e('books', 'blackhole-pro'); ?></a>. 
					<?php esc_html_e('Apply code', 'blackhole-pro'); ?> <code>PLANET24</code> <?php esc_html_e('at checkout. Sale ends 5/25/24.', 'blackhole-pro'); ?> 
					<?php echo blackhole_dismiss_notice_link(); ?>
				</p>
			</div>
			
			<?php
			
		}
		
	}
	
}

//

function blackhole_dismiss_notice_activate() {
	
	delete_option('blackhole-pro-dismiss-notice');
	
}

function blackhole_dismiss_notice_version() {
	
	$version_current = BBB_VERSION;
	
	$version_previous = get_option('blackhole-pro-dismiss-notice');
	
	$version_previous = ($version_previous) ? $version_previous : $version_current;
	
	if (version_compare($version_current, $version_previous, '>')) {
		
		delete_option('blackhole-pro-dismiss-notice');
		
	}
	
}

function blackhole_dismiss_notice_check() {
	
	$check = get_option('blackhole-pro-dismiss-notice');
	
	return ($check) ? true : false;
	
}

function blackhole_dismiss_notice_save() {
	
	if (isset($_GET['dismiss-notice-verify']) && wp_verify_nonce($_GET['dismiss-notice-verify'], 'blackhole_dismiss_notice')) {
		
		if (!current_user_can('manage_options')) exit;
		
		$result = update_option('blackhole-pro-dismiss-notice', BBB_VERSION, false);
		
		$result = $result ? 'true' : 'false';
		
		$location = admin_url('admin.php?page=blackhole_settings&dismiss-notice='. $result);
		
		wp_redirect($location);
		
		exit;
		
	}
	
}

function blackhole_dismiss_notice_link() {
	
	$nonce = wp_create_nonce('blackhole_dismiss_notice');
	
	$href  = add_query_arg(array('dismiss-notice-verify' => $nonce), admin_url('admin.php?page=blackhole_settings'));
	
	$label = esc_html__('Dismiss', 'blackhole-pro');
	
	echo '<a class="bbb-dismiss-notice" href="'. esc_url($href) .'">'. esc_html($label) .'</a>';
	
}

function blackhole_check_date_expired() {
	
	$expires = apply_filters('blackhole_check_date_expired', '2024-05-25');
	
	return (new DateTime() > new DateTime($expires)) ? true : false;
	
}

//

function blackhole_reset_options() {
	
	if (isset($_GET['reset-options-verify']) && wp_verify_nonce($_GET['reset-options-verify'], 'blackhole_reset_options')) {
		
		if (!current_user_can('manage_options')) exit;
		
		$options_delete = delete_option('blackhole_pro_options');
		
		$result = ($options_delete) ? 'true' : 'false';
		
		do_action('blackhole_reset_options', $options_delete);
		
		$location = admin_url('admin.php?page=blackhole_settings&reset-options='. $result);
		
		wp_redirect($location);
		
		exit;
		
	}
	
}

function blackhole_reset_badbots() { 
	
	if (isset($_GET['reset-badbots-verify']) && wp_verify_nonce($_GET['reset-badbots-verify'], 'blackhole_reset_badbots')) {
		
		if (!current_user_can('manage_options')) exit;
		
		$badbots_delete = delete_option('blackhole_pro_badbots');
		
		blackhole_clear_cache();
		
		blackhole_cron_deactivation();
		
		$result = ($badbots_delete) ? 'true' : 'false';
		
		do_action('blackhole_reset_badbots', $badbots_delete);
		
		$location = admin_url('admin.php?page=blackhole_settings&reset-badbots='. $result);
		
		wp_redirect($location);
		
		exit;
		
	}
	
}

function blackhole_reset_reports() {
	
	if (isset($_GET['reset-reports-verify']) && wp_verify_nonce($_GET['reset-reports-verify'], 'blackhole_reset_reports')) {
		
		if (!current_user_can('manage_options')) exit;
		
		global $blackhole_badbots;
		
		foreach ($blackhole_badbots as $key => $value) {
			
			if ($key) unset($blackhole_badbots[$key]['reported']);
			
		}
		
		$badbots_update = update_option('blackhole_pro_badbots', $blackhole_badbots, true);
		
		$result = ($badbots_update) ? 'true' : 'false';
		
		do_action('blackhole_reset_reports', $badbots_update);
		
		$location = admin_url('admin.php?page=blackhole_settings&reset-reports='. $result);
		
		wp_redirect($location);
		
		exit;
		
	}
	
}

function blackhole_refresh_geo() { 
	
	if (
		isset($_GET['blackhole-bot-id']) && isset($_GET['blackhole-ip-address']) && 
		isset($_GET['blackhole-geo-lookup']) && wp_verify_nonce($_GET['blackhole-geo-lookup'], 'blackhole_do_geo_lookup')
	) {
		
		if (!current_user_can('manage_options')) exit;
		
		global $blackhole_badbots;
		
		$bot_id     = sanitize_text_field($_GET['blackhole-bot-id']);
		$ip_address = sanitize_text_field($_GET['blackhole-ip-address']);
		
		$geo = blackhole_get_geo($ip_address, true);
		
		list ($code, $country, $region, $city, $zip) = $geo;
		
		foreach($blackhole_badbots as $key => $value) {
			
			if (isset($value['bot_id']) && $value['bot_id'] == $bot_id) {
				
				$blackhole_badbots[$key]['code']    = (!empty($code))    ? $code    : $blackhole_badbots[$key]['code'];
				$blackhole_badbots[$key]['country'] = (!empty($country)) ? $country : $blackhole_badbots[$key]['country'];
				$blackhole_badbots[$key]['region']  = (!empty($region))  ? $region  : $blackhole_badbots[$key]['region'];
				$blackhole_badbots[$key]['city']    = (!empty($city))    ? $city    : $blackhole_badbots[$key]['city'];
				$blackhole_badbots[$key]['zip']     = (!empty($zip))     ? $zip     : $blackhole_badbots[$key]['zip'];
				
				break;
				
			}
			
		}
		
		$badbots_refresh = update_option('blackhole_pro_badbots', $blackhole_badbots, true);
		
		$result = ($badbots_refresh) ? 'true' : 'false';
		
		$location = admin_url('admin.php?page=blackhole_badbots&refresh-geo='. $result .'&bot-id='. $bot_id);
		
		wp_redirect($location);
		
		exit;
		
	}
	
}

function blackhole_delete_bot() {
	
	global $blackhole_options, $blackhole_badbots;
	
	$updated_options = $blackhole_options;
	
	$updated_badbots = $blackhole_badbots;
	
	$whitelist_ips = isset($updated_options['ip_whitelist']) ? trim($updated_options['ip_whitelist'], ', ') : '';
	
	$whitelist_uas = isset($updated_options['bot_whitelist']) ? trim($updated_options['bot_whitelist'], ', ') : '';
	
	$verify = isset($_POST['update-bots-verify']) ? wp_verify_nonce($_POST['update-bots-verify'], 'blackhole_update_bot') : false;
	
	$is_delete = isset($_POST['delete-bots']) ? $_POST['delete-bots'] : false;
	
	$is_wlist_ip = isset($_POST['wlist-bots-ip']) ? $_POST['wlist-bots-ip'] : false;
	
	$is_wlist_ua = isset($_POST['wlist-bots-ua']) ? $_POST['wlist-bots-ua'] : false;
	
	if ($verify) {
		
		if (!current_user_can('manage_options')) exit;
		
		if (isset($_POST['bot-id'])) {
			
			$bot_ids = $_POST['bot-id'];
			
			foreach ($updated_badbots as $key => $value) {
				
				foreach ($bot_ids as $bot_id) {
					
					if ($bot_id === '' || !is_numeric($bot_id)) continue;
					
					if (intval($bot_id) === intval($value['bot_id'])) {
						
						if ($is_wlist_ip) {
							
							$ip = isset($value['ip_address']) ? sanitize_text_field($value['ip_address']) : '';
							
							$whitelist_ips .= "\n" . $ip;
							
						} elseif ($is_wlist_ua) {
							
							$ua = isset($value['user_agent']) ? sanitize_text_field($value['user_agent']) : '';
							
							$whitelist_uas .= ', '. $ua;
							
						}
						
						unset($updated_badbots[$key]);
						
					}
					
				}
				
			}
			
		}
		
		if (isset($updated_options['ip_whitelist']))  $updated_options['ip_whitelist']  = trim($whitelist_ips, ', ');
		
		if (isset($updated_options['bot_whitelist'])) $updated_options['bot_whitelist'] = trim($whitelist_uas, ', ');
		
		$options_update = update_option('blackhole_pro_options', $updated_options, true);
		
		$badbots_update = update_option('blackhole_pro_badbots', $updated_badbots, true);
		
		blackhole_clear_cache();
		
		if ($options_update && $badbots_update) $result = 'both';
		
		elseif ($options_update) $result = 'whitelist';
		
		elseif ($badbots_update) $result = 'delete';
		
		else $result = 'false';
		
		$filter = isset($_GET['filter']) ? urlencode($_GET['filter']) : 'bot_id';
		
		$page = isset($_GET['p']) ? urlencode($_GET['p']) : 1;
		
		$params = array('p' => $page, 'filter' => $filter, 'update-bot' => $result);
		
		$path = admin_url('admin.php?page=blackhole_badbots');
		
		do_action('blackhole_delete_bot', $bot_ids, $updated_badbots, $options_update, $badbots_update);
		
		$location = add_query_arg($params, $path);
		
		wp_redirect($location);
		
		exit;
		
	}
	
}

function blackhole_add_bot() {
	
	global $blackhole_badbots;
	
	$updated_badbots = $blackhole_badbots;
	
	if (isset($_POST['add-bots-verify']) && wp_verify_nonce($_POST['add-bots-verify'], 'blackhole_add_bot')) {
		
		if (!current_user_can('manage_options')) exit;
		
		if (isset($_POST['ip_address']) && !empty($_POST['ip_address'])) {
			
			$ip_address = sanitize_text_field($_POST['ip_address']);
			
			$request_uri = (isset($_POST['request_uri']))      ? sanitize_text_field($_POST['request_uri'])      : '';
			$remote_host = (isset($_POST['remote_host']))      ? sanitize_text_field($_POST['remote_host'])      : '';
			$user_agent  = (isset($_POST['user_agent']))       ? sanitize_text_field($_POST['user_agent'])       : '';
			$referrer    = (isset($_POST['referrer']))         ? sanitize_text_field($_POST['referrer'])         : '';
			$protocol    = (isset($_POST['protocol']))         ? sanitize_text_field($_POST['protocol'])         : '';
			$method      = (isset($_SERVER['REQUEST_METHOD'])) ? sanitize_text_field($_SERVER['REQUEST_METHOD']) : '';
			
			if (empty($remote_host)) $remote_host = sanitize_text_field(gethostbyaddr($ip_address));
			
			$date = blackhole_get_date();
			
			$bot_id = blackhole_get_bot_id();
			
			$geo = blackhole_get_geo($ip_address);
			
			list ($code, $country, $region, $city, $zip) = $geo;
			
			$add_bot = array(
				
				'bot_id'      => $bot_id,
				'ip_address'  => $ip_address,
				'request_uri' => $request_uri,
				'remote_host' => $remote_host,
				'user_agent'  => $user_agent,
				'referrer'    => $referrer,
				'protocol'    => $protocol,
				'method'      => $method,
				'date'        => $date,
				'code'        => $code,
				'country'     => $country,
				'region'      => $region,
				'city'        => $city,
				'zip'         => $zip,
				'hits'        => 1,
				'latest'      => $date,
				
			);
			
			array_push($updated_badbots, $add_bot);
			
			if (blackhole_check_log($add_bot)) {
				
				$result = 'exists';
				
			} else {
				
				$badbots_update = update_option('blackhole_pro_badbots', $updated_badbots, true);
				
				$result = ($badbots_update) ? 'true' : 'false';
				
			}
			
		} else {
			
			$ip_address = 'null';
			$bot_id     = 'null';
			$result     = 'ip-required';
			
		}
		
		$params = array('add-badbot' => $result, 'bot-id' => $bot_id, 'bot-ip' => $ip_address);
		
		$path = admin_url('admin.php?page=blackhole_badbots');
		
		do_action('blackhole_add_bot', $ip_address, $add_bot, $updated_badbots, $result);
		
		$location = add_query_arg($params, $path);
		
		wp_redirect($location);
		
		exit;
		
	}
	
}