<?php // Blackhole Pro - License & Activation

if (!defined('ABSPATH')) exit;

if (!class_exists('EDD_SL_Plugin_Updater')) {
	
	include(BBB_DIR .'updates/updater.php');
	
}



function blackhole_plugin_updater() {
	update_option('blackhole_pro_license_key', '123123123');
	update_option('blackhole_pro_license_status', 'valid');
	if (!current_user_can('manage_options') && !wp_doing_cron()) return;
	
	$license_key = trim(get_option('blackhole_pro_license_key'));
	
	$license_key = (defined('BLACKHOLE_LICENSE') && BLACKHOLE_LICENSE) ? BLACKHOLE_LICENSE : $license_key;
	
	$edd_updater = new EDD_SL_Plugin_Updater(
		
		BBB_HOME, 
		BBB_FILE, 
		array(
			'license'   => $license_key,
			'item_name' => BBB_NAME,
			'author'    => BBB_AUTHOR,
			'version'   => BBB_VERSION,
			'url'       => BBB_HOME,
			'beta'		=> false
			
		)
		
	);
	
}
add_action('admin_init', 'blackhole_plugin_updater', 0);



function blackhole_display_license() {
	
	$license = trim(get_option('blackhole_pro_license_key'));
	
	$license_constant = (defined('BLACKHOLE_LICENSE') && BLACKHOLE_LICENSE) ? BLACKHOLE_LICENSE : null;
	
	$status = get_option('blackhole_pro_license_status');
	
	$status_constant = get_option('blackhole_pro_license_constant');
	
	?>
	
	<div class="wrap">
		
		<h1 class="bbb-title"><?php echo BBB_NAME; ?> <span><?php echo BBB_VERSION; ?></span></h1>
		
		<?php $heading = ($license_constant) ? __('License Status', 'blackhole-pro') : __('Activate License', 'blackhole-pro'); ?>
		
		<h2><?php echo esc_html($heading); ?></h2>
		
		<p>
			<?php
				echo '<a target="_blank" rel="noopener noreferrer" href="https://plugin-planet.com/activate-deactivate-plugin-license/">';
				echo esc_html__('Activate your license', 'blackhole-pro') .'</a> ';
				echo esc_html__('to enable Blackhole Pro, 1-click updates, and plugin support. Visit the Help tab for more infos.', 'blackhole-pro');
			?>
		</p>
		
		<form method="post" action="options.php">
			
			<?php settings_fields('blackhole_license_settings'); ?>
			
			<?php if ($license_constant) : ?>
				
				<?php $class = ($status_constant === 'active') ? 'bbb-license-status-activated' : 'bbb-license-status-deactivated'; ?>
				
				<p><?php esc_html_e('License key added via', 'blackhole-pro'); ?> <code class="code-small">wp-config.php</code></p>
				<p><?php esc_html_e('Status:', 'blackhole-pro'); ?> <span class="bbb-license-constant <?php echo $class; ?>"><?php echo esc_html($status_constant); ?></span></p>
				
			<?php else : ?>
				
				<?php if (blackhole_check_license()) : ?>
					
					<p>
						<?php echo blackhole_obfuscate_password($license); ?>
						<label class="bbb-label bbb-license-label bbb-license-status-activated"><span class="emoji">✅</span> <?php esc_html_e('Your Blackhole Pro License is active.', 'blackhole-pro'); ?></label>
					</p>
					<p>
						<input type="submit" class="button button-primary" value="<?php esc_attr_e('Deactivate License', 'blackhole-pro'); ?>">
						<input type="hidden" name="blackhole_pro_license_key" value="<?php echo esc_attr($license); ?>">
						<?php wp_nonce_field('blackhole_license_deactivate', 'blackhole_license_deactivate'); ?>
					</p>
					
				<?php else : ?>
					
					<?php if (empty($license)) : ?>
						
						<p>
							<input type="password" class="bbb-license-input regular-text" id="blackhole_pro_license_key" name="blackhole_pro_license_key" value="<?php echo esc_attr($license); ?>">
							<?php echo blackhole_reveal_password(); ?>
							<label class="bbb-label bbb-license-label" for="blackhole_pro_license_key"><?php esc_html_e('Enter your License Key', 'blackhole-pro'); ?></label>
						</p>
						<p>
							<input type="submit" class="button button-primary" value="<?php esc_attr_e('Save License', 'blackhole-pro'); ?>">
							<?php wp_nonce_field('blackhole_license_save', 'blackhole_license_save'); ?>
						</p>
						
					<?php else : ?>
						
						<p>
							<input type="password" class="bbb-license-input regular-text" id="blackhole_pro_license_key" name="blackhole_pro_license_key" value="<?php echo esc_attr($license); ?>"> 
							<?php echo blackhole_clear_license_link(); ?>
							<label class="bbb-label bbb-license-label bbb-license-status-deactivated" for="blackhole_pro_license_key"><?php esc_html_e('Your license is inactive. Click the button to activate.', 'blackhole-pro'); ?></label>
						</p>
						<p>
							<input type="submit" class="button button-primary" value="<?php esc_attr_e('Activate License', 'blackhole-pro'); ?>">
							<?php wp_nonce_field('blackhole_license_activate', 'blackhole_license_activate'); ?>
						</p>
						
					<?php endif; ?>
					
				<?php endif; ?>
				
			<?php endif; ?>
			
		</form>
		
	</div>
	
	<?php
	
}



function blackhole_license_register_option() {
	
	register_setting('blackhole_license_settings', 'blackhole_pro_license_key', 'blackhole_sanitize_option');
}
add_action('admin_init', 'blackhole_license_register_option');



function blackhole_sanitize_option($new) {
	
	$old = trim(get_option('blackhole_pro_license_key'));
	
	if ($old && $old != $new) delete_option('blackhole_pro_license_status');
	
	return $new;
}



function blackhole_obfuscate_password($string) {
	
	$output = '';
	
	$length = strlen($string);
	
	$password = str_repeat('*', $length);
	
	$output = '<code class="bbb-license-key">'. esc_html($password) .'</code>';
	
	return $output;
	
}



function blackhole_reveal_password() {
	
	$title = esc_html__('Show License', 'blackhole-pro');
	
	return '<a class="bbb-reveal-password" href="#bbb-reveal-password" data-type="password" title="'. $title .'"><span class="fa fa-eye"></span></a>';
	
}



function blackhole_check_license() {
		
	$status = get_option('blackhole_pro_license_status');
	
	$status_constant = get_option('blackhole_pro_license_constant');
	
	if ($status === 'valid' || $status_constant === 'active') return true;
	
	return false;
	
}



function blackhole_license_status() {
	
	$output = '';
	
	$local = apply_filters('blackhole_license_status_local', false);
	
	if (!blackhole_check_license() && !$local) {
		
		$output .= '<p class="bbb-license-status">❗ ';
		
		$output .= esc_html__('Your Blackhole Pro license is not active.', 'blackhole-pro');
		
		$output .= ' <a href="'. esc_url(admin_url('admin.php?page=blackhole_license')) .'">'. esc_html__('Activate your license', 'blackhole-pro') .'</a> ';
		
		$output .= esc_html__('to enable plugin updates and support.', 'blackhole-pro');
		
		$output .= '</p>';
		
	}
	
	return $output;
	
}



function blackhole_get_server_response($action, $license_key = '') {
	
	$license_key = $license_key ? $license_key : get_option('blackhole_pro_license_key');
	
	$params = array('edd_action' => $action, 'license' => trim($license_key), 'item_name' => urlencode(BBB_NAME), 'url' => home_url());
	
	return wp_remote_get(BBB_HOME, array('timeout' => 15, 'sslverify' => false, 'body' => $params));
	
}



function blackhole_check_server_response($action, $license, $response) {
	
	if (empty($license)) return array(__('Home server not available.', 'blackhole-pro'), 'server_not_available');
	
	$error   = property_exists($license, 'error')   ? $license->error   : null;
	$expires = property_exists($license, 'expires') ? $license->expires : 'now';
	$success = property_exists($license, 'success') ? $license->success : false;
	$status  = property_exists($license, 'license') ? $license->license : false;
	
	$message = __('Unknown error.', 'blackhole-pro');
	
	if (wp_remote_retrieve_response_code($response) !== 200) {
		
		$message = __('Error making remote request. Please try again or contact Plugin Planet for help.', 'blackhole-pro');
		
	} elseif (is_wp_error($response)) {
		
		$message = $response->get_error_message() .'.';
		
	} else {
		
		if ($error) {
			
			$message = blackhole_license_error_message($error, $expires);
			
		} elseif ($success && $status) {
			
			if (
				($action === 'check_license'      && $status === 'valid') || 
				($action === 'activate_license'   && $status === 'valid') ||
				($action === 'deactivate_license' && $status === 'deactivated')
			) {
				
				$message = ''; // no error message
				
			} else {
				
				$message = blackhole_license_status_message($status, $expires);
				
			}
			
		} else {
			
			$message = blackhole_license_status_message($status, $expires);
			
		}
		
	}
	
	return array($message, $status);
	
}



function blackhole_redirect_server_response($action, $message, $status, $updated) {
	
	if (
		($action === 'activate_license'   && $status === 'valid') ||
		($action === 'deactivate_license' && $status === 'deactivated')
	) {
		
		$result = $updated ? 'true' : 'false';
		
	} else {
		
		$result = 'false';
	}
	
	$params = array($action => $result, 'license-response' => urlencode($message), 'license-status' => $status);
	
	$path = admin_url('admin.php?page=blackhole_license');
	
	$location = add_query_arg($params, $path);
	
	wp_redirect($location);
	
	exit;
	
}



function blackhole_test_license() {
	
	if (get_transient('blackhole_test_license') === false) {
		
		$action = 'check_license';
		
		$response = blackhole_get_server_response($action);
		
		$license = json_decode(wp_remote_retrieve_body($response));
		
		list ($message, $status) = blackhole_check_server_response($action, $license, $response);
		
		if (empty($message)) {
			
			$message = __('Blackhole Pro license test result: ', 'blackhole-pro') . $status;
			
		}
		
		update_option('blackhole_pro_license_status', $status);
		
		set_transient('blackhole_test_license', $status, DAY_IN_SECONDS);
		
		// error_log($message); // testing
		
	}
	
}
add_action('admin_init', 'blackhole_test_license');



function blackhole_activate_license() {
	
	if (isset($_POST['blackhole_license_activate']) && wp_verify_nonce($_POST['blackhole_license_activate'], 'blackhole_license_activate')) {
		
		if (!current_user_can('manage_options')) return;
		
		$updated = false;
		
		$action = 'activate_license';
		
		$response = blackhole_get_server_response($action);
		
		$license = json_decode(wp_remote_retrieve_body($response));
		
		list ($message, $status) = blackhole_check_server_response($action, $license, $response);
		
		if (empty($message) && $status === 'valid') {
			
			$message = __('License activated.', 'blackhole-pro');
			
			$updated = update_option('blackhole_pro_license_status', $status);
			
		}
		
		blackhole_redirect_server_response($action, $message, $status, $updated);
		
	}
	
}
add_action('admin_init', 'blackhole_activate_license');



function blackhole_deactivate_license() {
	
	if (isset($_POST['blackhole_license_deactivate']) && wp_verify_nonce($_POST['blackhole_license_deactivate'], 'blackhole_license_deactivate')) {
		
		if (!current_user_can('manage_options')) return;
		
		$updated = false;
		
		$action = 'deactivate_license';
		
		$response = blackhole_get_server_response($action);
		
		$license = json_decode(wp_remote_retrieve_body($response));
		
		list ($message, $status) = blackhole_check_server_response($action, $license, $response);
		
		if (empty($message) && $status === 'deactivated') {
			
			$message = __('License deactivated.', 'blackhole-pro');
			
			$updated = delete_option('blackhole_pro_license_status');
			
		}
		
		blackhole_redirect_server_response($action, $message, $status, $updated);
		
	}
	
}
add_action('admin_init', 'blackhole_deactivate_license');



function blackhole_config_constant_activate() {
	
	if (blackhole_get_current_screen_id() !== 'blackhole-pro_page_blackhole_license') return;
	
	if (isset($_POST['blackhole_license_activate']) || isset($_POST['blackhole_license_deactivate'])) return;
	
	if (isset($_GET['activate_license']) || isset($_GET['deactivate_license']) || isset($_GET['clear-license'])) return;
	
	if (!current_user_can('manage_options')) return;
	
	$license_key = (defined('BLACKHOLE_LICENSE') && BLACKHOLE_LICENSE) ? BLACKHOLE_LICENSE : null;
	
	if (empty($license_key)) {
		
		$updated = update_option('blackhole_pro_license_constant', 'inactive');
		
		$log = __('Blackhole Pro license wp-config constant not defined', 'blackhole-pro');
		
	} else {
		
		$action = $license_key ? 'activate_license' : 'deactivate_license';
		
		$response = blackhole_get_server_response($action, $license_key);
		
		$license = json_decode(wp_remote_retrieve_body($response));
		
		list ($message, $status) = blackhole_check_server_response($action, $license, $response);
		
		if ($message) { // error
			
			$updated = update_option('blackhole_pro_license_constant', 'inactive');
			
			$log = __('Blackhole Pro', 'blackhole-pro') .' '. $action .' '. __('failed due to error:', 'blackhole-pro') .' '. $message;
			
		} else { // success
			
			if ($status === 'valid') {
				
				$updated = update_option('blackhole_pro_license_constant', 'active');
				
				$log = __('Blackhole Pro license activated via wp-config constant', 'blackhole-pro');
				
			} elseif ($status === 'deactivated') {
				
				$updated = update_option('blackhole_pro_license_constant', 'inactive');
				
				$log = __('Blackhole Pro license deactivated via removed wp-config constant', 'blackhole-pro');
				
			} else {
				
				$updated = update_option('blackhole_pro_license_constant', 'inactive');
				
				$log = __('Blackhole Pro', 'blackhole-pro') .' '. $action .' '. __('failed due to error', 'blackhole-pro');
				
			}
			
		}
		
	}
	
	// if ($log) error_log($log); // testing
	
}
add_action('current_screen', 'blackhole_config_constant_activate');



function blackhole_clear_license_link() {
	
	$nonce = wp_create_nonce('blackhole_license_clear');
	
	$href  = esc_url(add_query_arg(array('blackhole_license_clear' => $nonce), admin_url('admin.php?page=blackhole_license')));
	
	$title = esc_html__('Clear license field', 'blackhole-pro');
	
	$label = '&#10005;';
	
	return '<a class="bbb-license-clear" href="'. $href .'" title="'. $title .'">'. $label .'</a>';
	
}



function blackhole_clear_license() {
	
	if (isset($_GET['blackhole_license_clear']) && wp_verify_nonce($_GET['blackhole_license_clear'], 'blackhole_license_clear')) {
		
		if (!current_user_can('manage_options')) return;
		
		$delete_key    = delete_option('blackhole_pro_license_key');
		
		$delete_status = delete_option('blackhole_pro_license_status');
		
		$result = ($delete_key) ? 'true' : 'false';
		
		$location = admin_url('admin.php?page=blackhole_license&clear-license='. $result);
		
		wp_redirect($location);
		
		exit;
		
	}
	
}
add_action('admin_init', 'blackhole_clear_license');



function blackhole_license_admin_notice() {
	
	if (blackhole_get_current_screen_id() === 'blackhole-pro_page_blackhole_license') {
		
		$response = isset($_GET['license-response']) ? urldecode($_GET['license-response']) : __('Undefined response.', 'blackhole-pro');
		
		$response .= ' <a target="_blank" rel="noopener noreferrer" href="'. esc_url(BBB_HOME .'/support/') .'">'. esc_html__('Get help&nbsp;&raquo;', 'blackhole-pro') .'</a>';
		
		if (isset($_GET['activate_license'])) {
			
			if ($_GET['activate_license'] === 'true') : ?>
				
				<div class="notice notice-success is-dismissible"><p><strong><?php esc_html_e('License activated.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php else : ?>
				
				<div class="notice notice-warning is-dismissible"><p><strong><?php echo esc_html__('License not activated:', 'blackhole-pro') .' '. $response; ?></strong></p></div>
				
			<?php endif;
			
		} elseif (isset($_GET['deactivate_license'])) {
			
			if ($_GET['deactivate_license'] === 'true') : ?>
				
				<div class="notice notice-success is-dismissible"><p><strong><?php esc_html_e('License deactivated.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php else : ?>
				
				<div class="notice notice-warning is-dismissible"><p><strong><?php echo esc_html__('License not deactivated:', 'blackhole-pro') .' '. $response; ?></strong></p></div>
				
			<?php endif;
			
		} elseif (isset($_GET['clear-license'])) {
			
			if ($_GET['clear-license'] === 'true') : ?>
				
				<div class="notice notice-success is-dismissible"><p><strong><?php esc_html_e('Done.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php else : ?>
				
				<div class="notice notice-info is-dismissible"><p><strong><?php esc_html_e('No changes made.', 'blackhole-pro'); ?></strong></p></div>
				
			<?php endif;
			
		}
		
	}
	
}
add_action('admin_notices', 'blackhole_license_admin_notice');



function blackhole_license_error_message($error, $expires) {
	
	switch($error) {
		
		case 'invalid' :
		case 'missing' :
			
			$message = __('Invalid license key. Please double-check and try again.', 'blackhole-pro');
			
			break;
			
		case 'disabled' :
		case 'revoked'  :
			
			$message = __('License key disabled. Contact Plugin Planet for help.', 'blackhole-pro');
			
			break;
			
		case 'expired' :
			
			$expires = date_i18n(get_option('date_format'), strtotime($expires, current_time('timestamp')));
			
			$message = __('Your license expired on', 'blackhole-pro') .' '. $expires .'. '. __('Renew your license for plugin updates and support.', 'blackhole-pro');
			
			break;
			
		case 'no_activations_left':
			
			$message = __('Your license key has reached its activation limit.', 'blackhole-pro');
			
			break;
			
		case 'site_inactive' :
			
			$message = __('Your license key is not active for this domain.', 'blackhole-pro');
			
			break;
			
		default :
			
			$message = __('An error occurred, please try again.', 'blackhole-pro');
			
			break;
		
	}
	
	return $message;
	
}



function blackhole_license_status_message($status, $expires) {
	
	switch($status) {
		
		case 'invalid' :
			
			$message = $message = __('Invalid license key. Please double-check and try again.', 'blackhole-pro');
			
			break;
			
		case 'disabled' :
			
			$message = $message = __('License key disabled. Contact Plugin Planet for help.', 'blackhole-pro');
			
			break;
			
		case 'expired' :
			
			$expires = date_i18n(get_option('date_format'), strtotime($expires, current_time('timestamp')));
			
			$message = __('Your license expired on', 'blackhole-pro') .' '. $expires .'. '. __('Renew your license for plugin updates and support.', 'blackhole-pro');
			
			break;
			
		case 'failed' :
			
			$message = __('License key action failed. Contact Plugin Planet for help.', 'blackhole-pro');
			
			break;
			
		default :
			
			$message = __('License key not valid for this plugin. Please double-check and try again.', 'blackhole-pro');
			
			break;
		
	}
	
	return $message;
	
}